!****h* src/integrals/two_point/generic_function_1p
! NAME
!
!  Module generic_function_1p
!
! USAGE
!
!  use generic_function_1p
!
! DESCRIPTION
!
!  This module contains the generic routines to compute 
!  one point functions in n  dimensions
!
! OUTPUT
!
!  It exports one public routine:
!  * f1p -- a function to compute the one point function in n dimensions
!
! USES
!
!  * precision (src/module/preci_double.f90)
!  * array (src/module/array.f90)
!  * logarithme (src/module/z_log.f90)
!  * tri_croissant (src/module/tri.f90)
!  * constante (src/module/constante.f90)
!  * sortie_erreur (src/module/sortie_erreur.f90)
!  * parametre (src/module/parametre.f90)
!
!*****
module generic_function_1p
  !
  use precision
  use array
  use logarithme
  use tri_croissant
  use constante
  use sortie_erreur
  use parametre
  implicit none
  !
  private
  !
  public :: f1p
  !
  contains
    !
    !****f* src/integrals/one_point/generic_function_1p/f1p
    ! NAME
    !
    !  Function f1p
    !
    ! USAGE
    !
    !  real_dim4 = f1p(s_mat,b_pro,parf1)
    !
    ! DESCRIPTION
    !
    !  This function computes the generic two point function in n dimensions, 
    !  with or without Feynman parameters in the numerator
    !
    ! INPUTS
    !
    !  * s_mat -- a real (type ki) array of rank 2, the S matrix
    !  * b_pro -- an integer which represents the set of the four unpinched
    !    propagators
    !  * parf1 -- an integer (optional), the label of the one Feynman parameter
    !
    ! SIDE EFFECTS
    !
    !  No side effect
    !
    ! RETURN VALUE
    !
    !  It returns a real (type ki) array of rank 1 and shape 4
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    function f1p(s_mat,b_pro,parf1)
      !
      real(ki), intent (in), dimension(:,:) :: s_mat
      integer, intent (in) :: b_pro
      integer, intent (in), optional :: parf1
      real(ki),dimension(4) :: f1p
      !
      !
      integer :: par1
      integer, dimension(1) :: z_param_ini,z_param_out
      real(ki) :: mass1
      integer :: m1
      integer, dimension(1) :: s
      !
      if (present(parf1)) then ; par1 = parf1 ; else ; par1 = 0 ; endif
      !
      z_param_ini = (/ par1 /)
      !
      where (z_param_ini /= 0)
        !
        z_param_ini = locateb(z_param_ini,b_pro)
        !
      elsewhere
        !
        z_param_ini = 0
        !
      end where
      !
      if ( minval(z_param_ini) == -1 ) then
        !
        f1p = 0._ki
        !
      else
        !
        call tri_int(z_param_ini,z_param_out)
        !
        s = unpackb(b_pro,countb(b_pro))
        !
        m1 = s(1)
      !
        mass1 = -s_mat(m1,m1)/2._ki
        !
        if ( (mass1 == zero) ) then
          !
          f1p = 0._ki
      !   call print_error('In function f1p:')
      !   call print_error('error: no one point function with mass = 0')
          !
        else
          !
          if  (z_param_out(1) == 0) then
	 ! 
            f1p(1) = 1._ki*mass1
            f1p(2) = 0._ki
            !
            if (rat_or_tot_par == 'tot') then
              !
              f1p(3) = mass1*(1._ki-real(z_log(mass1/mu2_scale_par,-1._ki)))
              f1p(4) = -mass1*aimag(z_log(mass1/mu2_scale_par,-1._ki))
              !
            else if (rat_or_tot_par == 'rat') then
              !
              f1p(3) = mass1*1._ki
              f1p(4) = 0._ki
              !
            end if
            !
          else if  (z_param_out(1) /= 0) then
            !
              f1p(1) = 0._ki
              f1p(2) = 0._ki
              f1p(3) = 0._ki
              f1p(4) = 0._ki
                !
          end if
          !
        end if
        !
      end if
      !
    end function f1p
    !
    !
end module generic_function_1p
