!
!****h* src/integrals/three_point/func_he
! NAME
!
!  Module func_he
!
! USAGE
!
!  use func_he
!
! DESCRIPTION
!
!  This module contains several functions for the computation of
!  int^1_0 dy y^(n-1)/(y*z1+(1-y)*z3) where z1 and z3 are complex numbers
!
! OUTPUT
!
!  This modules exports three functions:
!  * he -- a function
!  * he_gen -- a function
!  * he_c -- a function
!
! USES
!
!  * precision (src/module/preci_double.f90)
!  * numerical_evaluation (src/numerical/mod_numeric.f90)
!  * sortie_erreur (src/module/sortie_erreur.f90)
!  * parametre (src/module/parametre.f90)
!  * logarithme (src/module/z_log.f90)
!  * constante (src/module/constante.f90)
!
!*****
module func_he
  !
  use precision
  use numerical_evaluation
  use sortie_erreur, only : tab_erreur_par,catch_exception,origine_info_par,num_grand_b_info_par,denom_grand_b_info_par
  use parametre, only : coupure_3p2m,rat_or_tot_par,tolerance,alpha_par,beta_par,lambda_par
  use logarithme, only : z_log
  use constante, only : i_,un
  implicit none
  !
  private
  real(ki) :: a1_glob,a3_glob,eps_glob
  real(ki) :: plus_grand_glob
  integer :: expo_glob
  public :: he,he_gen,he_c
  !
  contains
    !
    !****f* src/integrals/three_point/func_he/he
    ! NAME
    !
    !  Function he
    !
    ! USAGE
    !
    !  real_dim2 = he(n,a1,a3)
    !
    ! DESCRIPTION
    !
    !  This function computes:
    !  int^1_0 dy y^(n-1)/(y*z1+(1-y)*z3)
    !  where z1 = -a1 -i lambda and z3 = -a3 - i lambda
    !  For n=1, it is equal to: (ln(z1)-ln(z3))/(z1-z3)
    !  compatible with the definition of HnE
    !  It switches to numerical evaluation if 
    !  |a1-a3|/max(|a1|,|a3|) < coupure_3p2m
    !
    ! INPUTS
    !
    !  * n -- an integer, the power of y in the integrand
    !  * a1 -- a real (type ki), the real part of z1 (time -1)
    !  * a3 -- a real (type ki), the real part of z3 (time -1)
    !
    ! SIDE EFFECTS
    !
    !  No side effect, the returned value depends on the global variables
    !  rat_or_tot_par, coupure_3p2m
    !
    ! RETURN VALUE
    !
    !  It returns a real (type ki) array of rank 1 and shape 2
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    function he(n,a1,a3)
      !
      integer, intent(in) :: n
      real(ki), intent(in) :: a1,a3
      real(ki), dimension(2) :: he
      !
      complex(ki) :: rest
      complex(ki) :: abserr
      complex(ki), dimension(4) :: ver
      real(ki) :: g1,g3
      !
      plus_grand_glob = max(abs(a1),abs(a3))
      g1 = a1/plus_grand_glob
      g3 = a3/plus_grand_glob
      ! les variables a1_glob, a3_glob, expo_glob et eps_glob sont globales 
      a1_glob = -g1
      a3_glob = -g3
      expo_glob = n
      ! on choisit eps_glob de telle facon que le pole soit hors du contour
      eps_glob = -sign(un,a1-a3)
      !
      ! mettre une coupure d'ordre 1 !!!!!
      if (abs(g1-g3) > coupure_3p2m) then
        !
        ver = 0._ki
        !
        if (n >= 1) then
          !
          if (rat_or_tot_par == 'rat') then
            !
            ver(1) = 0._ki
            !
          else if (rat_or_tot_par == 'tot') then
            !
            ver(1) = (z_log(-g1,-1._ki)-z_log(-g3,-1._ki))/(g1-g3)
            !
          end if
          !
        end if
        !
        if (n >= 2) then
          !
          ver(2) = (-g3*ver(1)+1._ki)/(g1-g3)
          !
        end if
        !
        if (n >= 3) then
          !
          ver(3) = (-g3*ver(2)+1._ki/2._ki)/(g1-g3)
          !
        end if
        !
        if (n >= 4) then
          !
          ver(4) = (-g3*ver(3)+1._ki/3._ki )/(g1-g3)
          !
        end if
        !
        he(1) = real(ver(n),ki)/plus_grand_glob
        he(2) = aimag(ver(n))/plus_grand_glob
        !
      else if ( (abs(g1-g3) <= coupure_3p2m) .and. &
                (rat_or_tot_par == 'tot') ) then
        !
        origine_info_par = "he"
        num_grand_b_info_par = n
        denom_grand_b_info_par = abs(a1-a3)
        !
        call generic_eval_numer(eval_numer_he,0._ki,1._ki,tolerance,rest,abserr)
        !
        tab_erreur_par(1)%a_imprimer = .true.
        tab_erreur_par(1)%chaine = 'In function he:'
        tab_erreur_par(2)%a_imprimer = .true.
        tab_erreur_par(2)%chaine = 'the error returned by adapt_gauss1 is: %z0'
        tab_erreur_par(2)%arg_comp = abserr
        call catch_exception(1)
        !
        he(1) = real(rest,ki)/plus_grand_glob
        he(2) = aimag(rest)/plus_grand_glob
        !
      else
        !
        tab_erreur_par(1)%a_imprimer = .true.
        tab_erreur_par(1)%chaine = 'In function he (file mod_he.f90)'
        tab_erreur_par(2)%a_imprimer = .true.
        tab_erreur_par(2)%chaine = 'the choice rat has been made, it is singular when a1=a3 %d0'
        tab_erreur_par(2)%arg_real=abs(g1-g3)
        call catch_exception(0)
        !
      end if
      !
    end function he
    !
    !****if* src/integrals/three_point/func_he/eval_numer_he
    ! NAME
    !
    !  Function eval_numer_he
    !
    ! USAGE
    !
    !  complex = eval_numer_he(u)
    !
    ! DESCRIPTION
    !
    !  This is the integrand for the numerical evaluation of he
    !
    ! INPUTS
    !
    !  * u -- a real (type ki), the integration variable
    !
    ! SIDE EFFECTS
    !
    !  No side effect. The variables a1_glob, a3_glob, expo_glob and eps_glob
    !  are global in this module whereas variables lambda_par,beta_par,
    !  alpha_par are given by the module parametre
    !
    ! RETURN VALUE
    !
    !  It returns a complex (type ki)
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    function eval_numer_he(u)
      !
      real(ki), intent (in) :: u
      complex(ki) :: eval_numer_he
      !
      real(ki) :: x,y
      complex(ki) :: z,jacob
      !
      x = u
      y = -lambda_par*u**alpha_par*(1._ki-u)**beta_par
      jacob = 1._ki - eps_glob*i_*lambda_par*u**(alpha_par-1._ki)&
              *(1._ki-u)**(beta_par-1._ki)*(alpha_par*(1._ki-u)-beta_par*u)
      z = x + eps_glob*i_*y
      eval_numer_he = z**(expo_glob-1)/(z*a1_glob+(1._ki-z)*a3_glob)
      eval_numer_he = -eval_numer_he*jacob
      !
    end function eval_numer_he
    !
    !****f* src/integrals/three_point/func_he/he_c
    ! NAME
    !
    !  Function he_c
    !
    ! USAGE
    !
    !  complex = he_c(n,a1,a3)
    !
    ! DESCRIPTION
    !
    ! This function computes the same thing as he
    ! but it returns a complex instead of a real array of rank 1 and shape 2
    !
    ! INPUTS
    !
    !  * n -- an integer, the power of y in the integrand
    !  * a1 -- a real (type ki), the real part of z1 (time -1)
    !  * a3 -- a real (type ki), the real part of z3 (time -1)
    !
    ! SIDE EFFECTS
    !
    !  No side effect
    !
    ! RETURN VALUE
    !
    !  It returns a complex (type ki)
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    function he_c(n,a1,a3)
      !
      integer, intent(in) :: n
      real(ki), intent(in) :: a1,a3
      complex(ki) :: he_c
      !
      real(ki), dimension(2) :: temp
      !
      temp = he(n,a1,a3)
      he_c = cmplx(temp(1),temp(2),ki)
      !
    end function he_c
    !
    !****f* src/integrals/three_point/func_he/he_gen
    ! NAME
    !
    !  Function he_gen
    !
    ! USAGE
    !
    !  real_dim2 = he_gen(n,a1,b1,a3,b3)
    !
    ! DESCRIPTION
    !
    !  This function computes:
    !  int^1_0 dy y^n/(y*z1+(1-y)*z3)
    !  where z1 = a1 + i b1 and z3 = a3 + i b3
    !  For n=1, it is equal to: (ln(z1)-ln(z3))/(z1-z3)
    !  It switches to numerical evaluation if 
    !  |a1-a3|/max(|a1|,|a3|) < coupure_3p2m
    !
    ! INPUTS
    !
    !  * n -- an integer, the power of y in the integrand
    !  * a1 -- a real (type ki), the real part of z1 
    !  * b1 -- a real (type ki), the imaginary part of z1 
    !  * a3 -- a real (type ki), the real part of z3 
    !  * b3 -- a real (type ki), the imaginary part of z3 
    !
    ! SIDE EFFECTS
    !
    !  No side effect, the returned value depends on the global variables
    !  rat_or_tot_par, coupure_3p2m
    !
    ! RETURN VALUE
    !
    !  It returns a real (type ki) array of rank 1 and shape 2
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    function he_gen(n,a1,b1,a3,b3)
      !
      integer, intent(in) :: n
      real(ki), intent(in) :: a1,b1,a3,b3
      real(ki), dimension(2) :: he_gen
      !
      complex(ki) :: rest
      complex(ki) :: abserr
      complex(ki), dimension(4) :: ver
      !
      plus_grand_glob = max(abs(a1),abs(a3))
      ! les variables a1_gen_glob, a3_gen_glob, expo_gen_glob et eps_gen_glob
      ! sont globales
      a1_glob = a1/plus_grand_glob
      a3_glob = a3/plus_grand_glob
      expo_glob = n
      ! on choisit eps de telle facon que le pole soit hors du contour
      eps_glob = sign(un,b1*a3-b3*a1)
      !
      ! mettre une coupure d'ordre 1 !!!!!
      if (abs(a1_glob-a3_glob) > coupure_3p2m) then
        !
        ver(1) = (z_log(a1,b1)-z_log(a3,b3))/(a1-a3)
        ver(2) = (-a3*ver(1)+1._ki)/(a1-a3)
        ver(3) = (-a3*ver(2)+1._ki/2._ki)/(a1-a3)
        ver(4) = (-a3*ver(3)+1._ki/3._ki )/(a1-a3)
        !
        rest = ver(n)
        !
      else
        !
        origine_info_par = "he_gen"
        num_grand_b_info_par = n
        denom_grand_b_info_par = abs(a1-a3)
        !
        call generic_eval_numer(eval_numer_he_gen,0._ki,1._ki,tolerance,rest,abserr)
        !
        tab_erreur_par(1)%a_imprimer = .true.
        tab_erreur_par(1)%chaine = 'In function he_gen:'
        tab_erreur_par(2)%a_imprimer = .true.
        tab_erreur_par(2)%chaine = 'the error returned by adapt_gauss1 is: %z0'
        tab_erreur_par(2)%arg_comp = abserr
        call catch_exception(1)
        !
      end if
      !
      he_gen(1) = real(rest,ki)/plus_grand_glob
      he_gen(2) = aimag(rest)/plus_grand_glob
      !
    end function he_gen
    !
    !****if* src/integrals/three_point/func_he/eval_numer_he_gen
    ! NAME
    !
    !  Function eval_numer_he_gen
    !
    ! USAGE
    !
    !  complex = eval_numer_he_gen(u)
    !
    ! DESCRIPTION
    !
    !  This is the integrand for the numerical evaluation of he_gen
    !
    ! INPUTS
    !
    !  * u -- a real (type ki), the integration variable
    !
    ! SIDE EFFECTS
    !
    !  No side effect. The variables a1_glob, a3_glob, expo_glob and eps_glob
    !  are global in this module whereas variables lambda_par,beta_par,
    !  alpha_par are given by the module parametre
    !
    ! RETURN VALUE
    !
    !  It returns a complex (type ki)
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    function eval_numer_he_gen(u)
      !
      real(ki), intent (in) :: u
      complex(ki) :: eval_numer_he_gen
      !
      real(ki) :: x,y
      complex(ki) :: z,jacob
      !
      x = u
      y = -lambda_par*u**alpha_par*(1._ki-u)**beta_par
      jacob = 1._ki - eps_glob*i_*lambda_par*u**(alpha_par-1._ki)&
              *(1._ki-u)**(beta_par-1._ki)*(alpha_par*(1._ki-u)-beta_par*u)
      z = x + eps_glob*i_*y
      eval_numer_he_gen = z**(expo_glob-1)/(z*a1_glob+(1._ki-z)*a3_glob)
      eval_numer_he_gen = eval_numer_he_gen*jacob
      !
    end function eval_numer_he_gen
    !
end module func_he
