!~ changed 13.5.2010 to include scale (mu^2)^eps
!~ the default scale is 1, defined in parametre.f90 
!****h* src/integral/two_point/function_2p_m1m2
! NAME
!
!  Module function_2p_m1m2
!
! USAGE
!
!  use function_2p_m1m2
!
! DESCRIPTION
!
!  This module is used to compute the two-point function
!  I_2(s,m1^2,m2^2)
!  with/without Feynman parameters in n dimensions
!
! OUTPUT
!
!  This module exports the functions:
!  * f2p_m1m2 -- a function for the computation of 
!  two-point integrals
!  with non-zero momentum and two masses: I2^n({zj})(s,m1^2,m2^2)
!  with/without Feynman parameters, in n dimensions
!  one of the masses can be zero
!  massless case is already contained in generic_function_2p
!
!  
!  i2sm1m2: computes the scalar two point function
!  where both propagators have nonzero mass: 
!  I_2^n(s,m1^2,m2^2)
!
!  i2sm1: computes the scalar two point function
!  where only one propagator has nonzero mass: 
!  I_2^n(s,m^2,0)
!
!
! USES
!
!  * precision (src/module/preci_double.f90)
!  * logarithme (src/module/z_log.f90)
!  * sortie_erreur (src/module/sortie_erreur.f90)
!  * function_2p0m_mi (src/integrals/two_point/function_2p0m_mi.f90)
!
!*****
module function_2p_m1m2
  !
  use precision
  use logarithme
  use sortie_erreur, only : tab_erreur_par,catch_exception
  use constante
  use equal
  use function_2p0m_1mi
  use parametre, only : rat_or_tot_par,mu2_scale_par
  !
  implicit none
  !
  private 
  !
  public :: f2p_m1m2, i2sm1m2, i2sm1
  !
  contains
    !
    !
    !****f* src/integral/two_point/function_2p_m1m2/f2p_m1m2
    ! NAME
    !
    !  Function f2p_m1m2
    !
    ! USAGE
    !
    !  real_dim6 = f2p_m1m2(msq,par1,par2)
    !
    ! DESCRIPTION
    !
    !  This function computes the 
    !  two point function in n dimensions
    !  with non-zero momentum and two massive propagators
    !  with up to two Feynman parameters in the numerator.
    !  It retuns an array of 4 reals corresponding to the real/imaginary
    !  part of the coefficient of the 
    !  1/epsilon term and the real/imaginary part of the 
    !  constant term.
    !  corresponds to eqs.(A.5),(A.7) in hep-ph/0504267
    !  note that for rank one A_j^{2,1}=MINUS I_2(j,...)
    !
    ! INPUTS
    !
    !  * m1_sq -- real (type ki), the value of the mass
    !  * par1 -- an integer, the label of one Feynman parameter
    !  * par2 -- an integer, the label of the second Feynman parameter
    !  Note that par1,par2 are ordered internally, i.e.
    !  par1 <= par2, note also to use zero for par1, par2 
    !  if this Feynman parameter does not exist.
    !  Use the routine tri_int(t_in,t_out) to order the labels in the module 
    !  tri_croissant (src/module/tri.f90)
    !
    ! SIDE EFFECTS
    !
    !  No side effect
    !
    ! RETURN VALUE
    !
    !  An real (type ki) array of rank 1 and shape 4 corresponding to 
    !  the real/imaginary part of the coefficient of the 1/epsilon term
    !  and the real/imaginary part of the constant term.
    !
    ! EXAMPLE
    !
    ! light-like-momentum two point function without Feynman parameters 
    ! f2p_m1m2(s,m1sq,m2sq,0,0) 
    ! with one Feynman parameter in the numerator z_1 
    ! f2p_m1m2(s,m1sq,m2sq,0,1)
    ! with two Feynman parameters in the numerator z_2^2
    ! f2p_m1m2(s,m1sq,m2sq,2,2) 
    ! with two Feynman parameters in the numerator z1*z_2
    ! f2p_m1m2(s,m1sq,m2sq,1,2) 
    !
    !***** 
    function f2p_m1m2(s,m1,m2,par1,par2)
      ! m1 and m2 are the squared masses
      ! should only be called if s is nonzero
      !
      real(ki), intent (in) :: s,m1,m2
      integer, intent (in) :: par1,par2
      real(ki), dimension(4) :: f2p_m1m2, i2sca
      !
      f2p_m1m2 = 0.d0
      i2sca=i2sm1m2(s,m1,m2)
      !
      ! scalar case
      if ( (par1 == 0) .and. (par2 == 0) ) then
        !
	if ( .not.(equal_real(m1,zero)) .and. .not.(equal_real(m2,zero)) ) then
	! 
            f2p_m1m2 = i2sm1m2(s,m1,m2)
        !
	else if (equal_real(m1,zero)) then
	!
            f2p_m1m2 = i2sm1(s,m2)
        !
	else if (equal_real(m2,zero)) then 
	!
            f2p_m1m2 = i2sm1(s,m1)
        !
	else 
	    !~ call print_error('problem with mass values in f2p_m1m2')
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'problem with mass values in f2p_m1m2:'
          call catch_exception(0)
	end if   ! end test if m1 or m2 zero
       ! rank one: note that rat or tot is distinguished in i2sm1m2
      else if ( (par1 == 0) .and. (par2 == 1) ) then
        !
        f2p_m1m2 =  i2sm1m2(s,m1,m2)/2._ki -   &
     &                 (m1-m2)/s/2._ki*( i2sm1m2(s,m1,m2) - i20m1m2(m1,m2) )
       !
      else if ( (par1 == 0) .and. (par2 == 2) ) then
        !
        f2p_m1m2 =  i2sm1m2(s,m1,m2)/2._ki + &
     &                  (m1-m2)/s/2._ki*( i2sm1m2(s,m1,m2) - i20m1m2(m1,m2) )
	!
	! rank two: rat singled out explicitly here
      else if ( (par1 == 1) .and. (par2 == 1) ) then
        !
        f2p_m1m2(1) = 1._ki/3._ki
        f2p_m1m2(2) = 0._ki
      !	
          if (rat_or_tot_par == 'rat') then
            !
        f2p_m1m2(3) = (-6*m1**2 + 12*m1*m2 - 6*m2**2 + 9*m1*s - 9*m2*s + s**2 + &
     &                     6*(m1**2 + m2**2 + m2*s + s**2 - 2*m1*(m2 + s))*               &
     &                     i2sca(3) )/(18*s**2)
      !
        f2p_m1m2(4) = ( 6*(m1**2 + m2**2 + m2*s + s**2 - 2*m1*(m2 + s))*   &
     &                   i2sca(4) )/(18*s**2)
	!
          else if (rat_or_tot_par == 'tot') then
      !
        f2p_m1m2(3) = (-6*m1**2 + 12*m1*m2 - 6*m2**2 + 9*m1*s - 9*m2*s + s**2 + &
     &                     6*(m1**2 + m2**2 + m2*s + s**2 - 2*m1*(m2 + s))*               &
     &                     i2sca(3) +                                            &
     &                    6*m1*(m1 - m2 - 2*s)*real(z_log(m1/mu2_scale_par,-1._ki)) -                      &
     &                    6*m1*m2*real(z_log(m2/mu2_scale_par,-1._ki)) +                                       &
     &                    6*m2**2*real(z_log(m2/mu2_scale_par,-1._ki)) + & 
     &                    6*m2*s*real(z_log(m2/mu2_scale_par,-1._ki)))/(18*s**2)
      !
        f2p_m1m2(4) = ( 6*(m1**2 + m2**2 + m2*s + s**2 - 2*m1*(m2 + s))*   &
     &                   i2sca(4) +                                  &
     &                   6*m1*(m1 - m2 - 2*s)*aimag(z_log(m1/mu2_scale_par,-1._ki)) -           &
     &                   6*m1*m2*aimag(z_log(m2/mu2_scale_par,-1._ki)) +   &
     &                   6*m2**2*aimag(z_log(m2/mu2_scale_par,-1._ki)) +     &
     &                   6*m2*s*aimag(z_log(m2/mu2_scale_par,-1._ki)))/(18*s**2)
	!
	   end if ! end if rat or tot
	!   
      else if ( (par1 == 1) .and. (par2 == 2) ) then
        !
        f2p_m1m2(1) = 1._ki/6._ki
        f2p_m1m2(2) = 0._ki
       !	
          if (rat_or_tot_par == 'rat') then
         !
        f2p_m1m2(3) = (6*m1**2 - 12*m1*m2 + 6*m2**2 - s**2 +   &
     &                    3*(-2*m1**2 - 2*m2**2 + m2*s + s**2 + m1*(4*m2 + s))*  &
     &                    i2sca(3) )/(18*s**2)
      !
        f2p_m1m2(4) = (3*(-2*m1**2 - 2*m2**2 + m2*s + s**2 + m1*(4*m2 + s))*  &
     &                    i2sca(4) )/(18*s**2)
	!
          else if (rat_or_tot_par == 'tot') then
      !
        f2p_m1m2(3) = (6*m1**2 - 12*m1*m2 + 6*m2**2 - s**2 +   &
     &                    3*(-2*m1**2 - 2*m2**2 + m2*s + s**2 + m1*(4*m2 + s))*  &
     &                    i2sca(3) +      &
     &                    3*m1*(-2*m1 + 2*m2 + s)*real(z_log(m1/mu2_scale_par,-1._ki)) +  &
     &                    6*m1*m2*real(z_log(m2/mu2_scale_par,-1._ki)) -  &
     &                    6*m2**2*real(z_log(m2/mu2_scale_par,-1._ki)) + &
     &                    3*m2*s*real(z_log(m2/mu2_scale_par,-1._ki)))/(18*s**2)
      !
        f2p_m1m2(4) = (3*(-2*m1**2 - 2*m2**2 + m2*s + s**2 + m1*(4*m2 + s))*  &
     &                    i2sca(4) +      &
     &                    3*m1*(-2*m1 + 2*m2 + s)*aimag(z_log(m1/mu2_scale_par,-1._ki)) +  &
     &                    6*m1*m2*aimag(z_log(m2/mu2_scale_par,-1._ki)) -  &
     &                    6*m2**2*aimag(z_log(m2/mu2_scale_par,-1._ki)) + &
     &                    3*m2*s*aimag(z_log(m2/mu2_scale_par,-1._ki)))/(18*s**2)
	!
	   end if ! end if rat or tot
	!   
      else if ( (par1 == 2) .and. (par2 == 2) ) then
        !
        f2p_m1m2(1) = 1._ki/3._ki
        f2p_m1m2(2) = 0._ki
	!
          if (rat_or_tot_par == 'rat') then
      !
       f2p_m1m2(3) = ( -6*m1**2 + 12*m1*m2 - 6*m2**2 - 9*m1*s + 9*m2*s + s**2 +  &
     &                      6*(m1**2 + (m2 - s)**2 + m1*(-2*m2 + s))*  &
     &                     i2sca(3) )/(18*s**2)
      !
        f2p_m1m2(4) = ( 6*(m1**2 + (m2 - s)**2 + m1*(-2*m2 + s))*  & 
     &                      i2sca(4) )/(18*s**2)
	!
          else if (rat_or_tot_par == 'tot') then
      !
       f2p_m1m2(3) = (-6*m1**2 + 12*m1*m2 - 6*m2**2 - 9*m1*s + 9*m2*s + s**2 +  &
     &                      6*(m1**2 + (m2 - s)**2 + m1*(-2*m2 + s))*i2sca(3) + &
     &                      6*m1*(m1 - m2 + s)*real(z_log(m1/mu2_scale_par,-1._ki)) -  &
     &                      6*m1*m2*real(z_log(m2/mu2_scale_par,-1._ki)) +   &
     &                      6*m2**2*real(z_log(m2/mu2_scale_par,-1._ki)) -   &
     &                      12*m2*s*real(z_log(m2/mu2_scale_par,-1._ki)))/(18*s**2)
      !
        f2p_m1m2(4) = ( 6*(m1**2 + (m2 - s)**2 + m1*(-2*m2 + s))*i2sca(4) + &
     &                      6*m1*(m1 - m2 + s)*aimag(z_log(m1/mu2_scale_par,-1._ki)) -  &
     &                      6*m1*m2*aimag(z_log(m2/mu2_scale_par,-1._ki)) +   &
     &                      6*m2**2*aimag(z_log(m2/mu2_scale_par,-1._ki)) -   &
     &                      12*m2*s*aimag(z_log(m2/mu2_scale_par,-1._ki)))/(18*s**2)
	!
    	   end if ! end if rat or tot
	!   
      end if
        !
      !
    end function f2p_m1m2
    !
    !
    !
    ! ****f* src/integral/two_point/i2sm1m2
    ! NAME
    !
    !  Function i2sm1m2
    !
    ! USAGE
    !
    !  complex = i2sm1m2(s,m1sq,m2sq)
    !
    ! DESCRIPTION
    !
    !  This function computes the scalar two point function
    !  with non-zero momentum and two nonzero masses: I_2(s,m1^2,m2^2)
    !  in n dimensions
    !
    ! INPUTS
    !
    !  * msq -- a real (type ki), the mass squared
    !
    ! SIDE EFFECTS
    !
    !  No side effect, it uses the value of rat_or_tot_par 
    !  (in src/module/parametre.f90)
    !
    ! RETURN VALUE
    !
    !  It returns a real (type ki) array of rank 1 and shape 4
    !
    !*****
    function i2sm1m2(s,m1,m2)
      !
      real(ki), intent(in) :: s,m1,m2
      real(ki), dimension(4) :: i2sm1m2
      real(ki) :: delta,sig,sig1,sig2
      complex(ki) :: i2fin,tlog1,tlog2,x1,x2,t1,t2,r1,r2,rlog1,rlog2
      !
      delta = s**2+m2**2+m1**2-2._ki*s*m2-2._ki*s*m1-2._ki*m2*m1    
      sig = sign(1.d0,s)
      !
      if (delta >= 0.d0) then
          !
          x1 = (s+m1-m2+sqrt(delta))/(2._ki*s)  ! Im x1 =sign(s)*i*eps 
          x2 = (s+m1-m2-sqrt(delta))/(2._ki*s)   ! Im x2 =-sign(s)*i*eps 
          !
        else if (delta < 0.d0) then
          !
          x1 = (s+m1-m2+(-sig*i_)*sqrt(-delta))/(2._ki*s)
          x2 = (s+m1-m2-(-sig*i_)*sqrt(-delta))/(2._ki*s)
	  !
        end if
	!
	  t1=(x1-1._ki)
	  t2=(x2-1._ki)
	  r1=(x1-1._ki)/x1
	  r2=(x2-1._ki)/x2
	  !
	  if ( .not.(equal_real(aimag(t1),zero))) then
	    tlog1= log(t1)
	  else  
	    tlog1=z_log(real(t1),1._ki*sig)
	  endif  
	!
	  if ( .not.(equal_real(aimag(t2),zero))) then
	    tlog2= log(t2)
	  else  
	     tlog2=z_log(real(t2),-1._ki*sig)
	  endif  
	!
	  if ( .not.(equal_real(aimag(x1),zero))) then
	    rlog1= log(x1)
	  else  
	    rlog1=z_log(real(x1),1._ki*sig)
	  endif  
	!
	  if ( .not.(equal_real(aimag(x2),zero))) then
	    rlog2= log(x2)
	  else  
	    rlog2=z_log(real(x2),-1._ki*sig)
	  endif  
	!
!	  write(6,*) 'delta=',delta
!	  write(6,*) 't1=',t1
!	  write(6,*) 't2=',t2
!	  write(6,*) 'tlog1=',tlog1
!	  write(6,*) 'tlog2=',tlog2
!
! ***** to be checked ! **************************
	!
          i2fin = 2._ki-z_log(s/mu2_scale_par,1._ki) + t1*tlog1-x1*rlog1 + t2*tlog2-x2*rlog2
      !
  ! *************************************************
      !
         i2sm1m2(1) = 1._ki
         i2sm1m2(2) = 0._ki
      !
      if (rat_or_tot_par == 'rat') then
        !
         i2sm1m2(3) = 2._ki
         i2sm1m2(4) = 0._ki
        !
      else if (rat_or_tot_par == 'tot') then
        !
         i2sm1m2(3) = real(i2fin)
         i2sm1m2(4) = aimag(i2fin)
        !
      end if
      !
   end function i2sm1m2
    !
    !
    ! ****f* src/integral/two_point/i2sm1
    ! NAME
    !
    !  Function i2sm1
    !
    ! USAGE
    !
    !  complex = i2sm1(s,msq)
    !
    ! DESCRIPTION
    !
    !  This function computes the scalar two point function
    !  with non-zero momentum and m2=0: I_2(s,m^2,0)
    !  in n dimensions
    !
    ! INPUTS
    !
    !  * msq -- a real (type ki), the mass squared
    !
    ! SIDE EFFECTS
    !
    !  No side effect, it uses the value of rat_or_tot_par 
    !  (in src/module/parametre.f90)
    !
    ! RETURN VALUE
    !
    !  It returns a real (type ki) array of rank 1 and shape 4
    !
    !*****
    function i2sm1(s1,m1s)
      !
      real(ki), intent(in) :: s1,m1s
      real(ki), dimension(4) :: i2sm1
      !
         i2sm1(1) = 1._ki
         i2sm1(2) = 0._ki
      !
      if (rat_or_tot_par == 'rat') then
        !
         i2sm1(3) = 2._ki
         i2sm1(4) = 0._ki
        !
      else if (rat_or_tot_par == 'tot') then
        !
         i2sm1(3) = real( -z_log(m1s/mu2_scale_par,-1._ki)+2._ki+ &
     &              (m1s-s1)/s1*( z_log((m1s-s1)/mu2_scale_par,-1._ki)  - z_log(m1s/mu2_scale_par,-1._ki) ) )
         i2sm1(4) = aimag( -z_log(m1s/mu2_scale_par,-1._ki)+2._ki+ &
     &              (m1s-s1)/s1*( z_log((m1s-s1)/mu2_scale_par,-1._ki)  - z_log(m1s/mu2_scale_par,-1._ki) ) )
        !
      end if
      !
   end function i2sm1
    !
    !
    !
end module function_2p_m1m2
