!****h* src/integrals/two_point/generic_function_2p
!~ changed 13.5.2010 to include scale (mu^2)^eps
!~ the default scale is 1, defined in parametre.f90 
! NAME
!
!  Module generic_function_2p
!
! USAGE
!
!  use generic_function_2p
!
! DESCRIPTION
!
!  This module contains the generic routines to compute the
!  two point functions in n and n+2 dimensions
!
! OUTPUT
!
!  It exports two public routine:
!  * f2p -- a function to compute the two point function in n dimensions
!  * f2p_np2 -- a function to compute the two point function in n+2 dimensions
!
! USES
!
!  * precision (src/module/preci_double.f90)
!  * array (src/module/array.f90)
!  * logarithme (src/module/z_log.f90)
!  * tri_croissant (src/module/tri.f90)
!  * constante (src/module/constante.f90)
!  * sortie_erreur (src/module/sortie_erreur.f90)
!  * parametre (src/module/parametre.f90)
!  * function_2p0m_mi (src/integrals/two_point/function_2p0m_mi.f90)
!  * function_2p_m1m2 (src/integrals/two_point/function_2p_m1m2.f90)
!
!*****
module generic_function_2p
  !
  use precision
  use array
  use logarithme
  use tri_croissant
  use constante
  use sortie_erreur, only : tab_erreur_par,catch_exception
  use parametre
  use equal
  use function_2p0m_1mi
  use function_2p_m1m2
  !
  implicit none
  !
  private
  !
  public :: f2p,f2p_np2
  !
  contains
    !
    !****f* src/integrals/two_point/generic_function_2p/f2p
    ! NAME
    !
    !  Function f2p
    !
    ! USAGE
    !
    !  real_dim4 = f2p(s_mat,b_pro,parf1,parf2)
    !
    ! DESCRIPTION
    !
    !  This function computes the generic two point function in n dimensions, 
    !  with or without Feynman parameters in the numerator
    !
    ! INPUTS
    !
    !  * s_mat -- a real (type ki) array of rank 2, the S matrix
    !  * b_pro -- an integer which represents the set of the four unpinched
    !    propagators
    !  * parf1 -- an integer (optional), the label of the one Feynman parameter
    !  * parf2 -- an integer (optional), the label of the second Feynman parameter
    !
    ! SIDE EFFECTS
    !
    !  No side effect
    !
    ! RETURN VALUE
    !
    !  It returns a real (type ki) array of rank 1 and shape 4
    !
    !
    !*****
    function f2p(s_mat,b_pro,parf1,parf2)
      !
      real(ki), intent (in), dimension(:,:) :: s_mat
      integer, intent (in) :: b_pro
      integer, intent (in), optional :: parf1,parf2
      real(ki),dimension(4) :: f2p,i2sonem
      !
      !
      integer :: par1,par2
      integer, dimension(2) :: z_param_ini,z_param_out
      real(ki) :: arg1,arg2,arg3
      real(ki) :: s12,mass1,mass2,diffm,cuts12
      integer :: m1,m2
      integer, dimension(2) :: s
      !
      ! cuts12=1.d-15
      !
      if (present(parf1)) then ; par1 = parf1 ; else ; par1 = 0 ; endif
      if (present(parf2)) then ; par2 = parf2 ; else ; par2 = 0 ; endif
      !
      z_param_ini = (/ par1,par2 /)
      !
      where (z_param_ini /= 0)
        !
        z_param_ini = locateb(z_param_ini,b_pro)
        !
      elsewhere
        !
        z_param_ini = 0
        !
      end where
      !
      if ( minval(z_param_ini) == -1 ) then
        !
        f2p = 0._ki
        !
      else
        !
        call tri_int(z_param_ini,z_param_out)
        !
        s = unpackb(b_pro,countb(b_pro))
        !
        m1 = s(1)
        m2 = s(2)
        !
        arg1 = s_mat(m1,m2)
        arg2 = s_mat(m1,m1)
        arg3 = s_mat(m2,m2)
        !
 ! internal masses	
        mass1 = -s_mat(m1,m1)/2._ki
        mass2 = -s_mat(m2,m2)/2._ki
        s12 = arg1+mass1+mass2
    !     call print_error('arg1=',arg_real=arg1)
   !     call print_error('arg2=',arg_real=arg2)
   !     call print_error('arg3=',arg_real=arg3)
   !	call print_error('s12=',arg_real=s12)
	diffm=mass1-mass2
!	
!       if ( (arg1 == zero) .and. (arg2 == zero) .and. (arg3 == zero) ) then
       if ( (equal_real(s12,zero)) .and. (arg2 == zero) .and. (arg3 == zero) ) then
          !
                f2p = 0._ki
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'In function f2p:'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'no two point function with external mass = 0'
          call catch_exception(0)
        !  call print_error('scaleless two-point function is zero')
	!
        else if ( .not.(equal_real(s12,zero)) .and. (equal_real(mass1,zero)) .and. (equal_real(mass2,zero)) ) then 
	  ! massless case
          !
          arg1 = arg1/mu2_scale_par
          !
          if ( (z_param_out(1) == 0) .and. (z_param_out(2) == 0) ) then
            !
            f2p(1) = 1._ki
            f2p(2) = 0._ki
            !
            if (rat_or_tot_par == 'tot') then
              !
              f2p(3) = 2._ki-real(z_log(-s12/mu2_scale_par,-1._ki))
              f2p(4) = -aimag(z_log(-s12/mu2_scale_par,-1._ki))
              !
            else if (rat_or_tot_par == 'rat') then
              !
              f2p(3) = 2._ki
              f2p(4) = 0._ki
              !
            end if
            !
          else if ( (z_param_out(1) == 0) .and. (z_param_out(2) /= 0) ) then
            !
            f2p(1) = 1._ki/2._ki
            f2p(2) = 0._ki
            !
            if (rat_or_tot_par == 'tot') then
              !
              f2p(3) = 1._ki-real(z_log(-s12/mu2_scale_par,-1._ki))/2._ki
              f2p(4) = -aimag(z_log(-s12/mu2_scale_par,-1._ki))/2._ki
              !
            else if (rat_or_tot_par == 'rat') then
              !
              f2p(3) = 1._ki
              f2p(4) = 0._ki
              !
            end if
            !
          else if ( (z_param_out(1) /= 0) .and. (z_param_out(2) /= 0) ) then
            !
            if (z_param_out(1) == z_param_out(2)) then
              !
              f2p(1) = 1._ki/3._ki
              f2p(2) = 0._ki
              !
              if (rat_or_tot_par == 'tot') then
                !
                f2p(3) = 13._ki/18._ki-real(z_log(-s12/mu2_scale_par,-1._ki))/3._ki
                f2p(4) = -aimag(z_log(-s12/mu2_scale_par,-1._ki))/3._ki
                !
              else if (rat_or_tot_par == 'rat') then
                !
                f2p(3) = 13._ki/18._ki
                f2p(4) = 0._ki
                !
              end if
              !
            else
              !
              f2p(1) = 1._ki/6._ki
              f2p(2) = 0._ki
              !
              if (rat_or_tot_par == 'tot') then
                !
                f2p(3) = 5._ki/18._ki-real(z_log(-s12/mu2_scale_par,-1._ki),ki)/6._ki
                f2p(4) = -aimag(z_log(-s12/mu2_scale_par,-1._ki))/6._ki
                !
              else if (rat_or_tot_par == 'rat') then
                !
                f2p(3) = 5._ki/18._ki
                f2p(4) = 0._ki
                !
              end if  ! end if rat or tot
              !
            end if ! end if z1==z2
            !
          end if  ! end test value of z1,z2
          !
	 !
	 !*************** massive cases *******************************
	 ! added 07.08.09
	 ! assumes real masses in numerator
	 ! ************************************************************
        else if (  (equal_real(s12,zero)) .and. (.not.(equal_real(mass1,zero))) .and. &
     &          (equal_real(mass2,zero)) ) then 
	! case p^2=0, m1 nonzero, m2=0
             ! write(6,*) 'case (2b): s12 =0, m2 =0, m1  nonzero'
       !
          if ( (z_param_out(1) == 0) .and. (z_param_out(2) == 0) ) then
            !
	       f2p = i20m1(mass1)
	    !   
          else  if ( (z_param_out(1) == 0) .and. (z_param_out(2) == 1) ) then
	    !
	       f2p(1) = 1._ki/2._ki
	       f2p(2) = 0._ki
	    !
	     if (rat_or_tot_par == 'rat') then
        !
               f2p(3) = 1._ki/4._ki
	       f2p(4) = 0._ki
         !
             else if (rat_or_tot_par == 'tot') then
        !
               f2p(3) = -(-1 + 2*real(z_log(mass1/mu2_scale_par,-1._ki)))/4._ki
	       f2p(4) = -aimag(z_log(mass1/mu2_scale_par,-1._ki))/2._ki
         !
             end if  ! end if rat or tot
        !
          else  if ( (z_param_out(1) == 0) .and. (z_param_out(2) == 2) ) then
	    !
	       f2p(1) = 1._ki/2._ki
	       f2p(2) = 0._ki
	    !
	     if (rat_or_tot_par == 'rat') then
        !
               f2p(3) = 3._ki/4._ki
	       f2p(4) = 0._ki
         !
             else if (rat_or_tot_par == 'tot') then
        !
               f2p(3) = -(-3 + 2*real(z_log(mass1/mu2_scale_par,-1._ki)))/4._ki
	       f2p(4) = -aimag(z_log(mass1/mu2_scale_par,-1._ki))/2._ki
       	    !
             end if  ! end if rat or tot
        !
          else  if ( (z_param_out(1) == 1) .and. (z_param_out(2) == 1) ) then
	    !
	       f2p(1) = 1._ki/3._ki
	       f2p(2) = 0._ki
	    !
	     if (rat_or_tot_par == 'rat') then
        !
               f2p(3) = 1._ki/9._ki
	       f2p(4) = 0._ki
         !
             else if (rat_or_tot_par == 'tot') then
        !
               f2p(3) = (1 - 3*real(z_log(mass1/mu2_scale_par,-1._ki)))/9._ki
	       f2p(4) =  - aimag(z_log(mass1/mu2_scale_par,-1._ki))/3._ki
       	    !
             end if  ! end if rat or tot
	    !
          else  if ( (z_param_out(1) == 1) .and. (z_param_out(2) == 2) ) then
	    !
	       f2p(1) = 1._ki/6._ki
	       f2p(2) = 0._ki
	    !
	     if (rat_or_tot_par == 'rat') then
        !
               f2p(3) = 5._ki/36._ki
	       f2p(4) = 0._ki
         !
             else if (rat_or_tot_par == 'tot') then
        !
               f2p(3) = (5 - 6*real(z_log(mass1/mu2_scale_par,-1._ki)))/36._ki
	       f2p(4) =  - aimag(z_log(mass1/mu2_scale_par,-1._ki))/6._ki
       	    !
             end if  ! end if rat or tot
	    !
          else  if ( (z_param_out(1) == 2) .and. (z_param_out(2) == 1) ) then
	    !
	       f2p(1) = 1._ki/6._ki
	       f2p(2) = 0._ki
	    !
	     if (rat_or_tot_par == 'rat') then
        !
               f2p(3) = 5._ki/36._ki
	       f2p(4) = 0._ki
         !
             else if (rat_or_tot_par == 'tot') then
        !
               f2p(3) = (5 - 6*real(z_log(mass1/mu2_scale_par,-1._ki)))/36._ki
	       f2p(4) =  - aimag(z_log(mass1/mu2_scale_par,-1._ki))/6._ki
       	    !
             end if  ! end if rat or tot
	    !
          else  if ( (z_param_out(1) == 2) .and. (z_param_out(2) == 2) ) then
	    !
	       f2p(1) = 1._ki/3._ki
	       f2p(2) = 0._ki
	    !
	     if (rat_or_tot_par == 'rat') then
        !
               f2p(3) = 11._ki/18._ki
	       f2p(4) = 0._ki
         !
             else if (rat_or_tot_par == 'tot') then
        !
               f2p(3) = (11 - 6*real(z_log(mass1/mu2_scale_par,-1._ki)))/18._ki
	       f2p(4) =  - aimag(z_log(mass1/mu2_scale_par,-1._ki))/3._ki
       	    !
             end if  ! end if rat or tot
	    !
          end if  ! end test value of z1,z2
          !
	 ! ******************
        else if (  (equal_real(s12,zero)) .and. (equal_real(mass1,zero))    &
     &       .and.(.not.(equal_real(mass2,zero))) ) then 
	! case p^2=0, m2 nonzero, m1=0
        	 ! write(6,*) 'case (2a) s12 =0, m1 =0, m2  nonzero'
    !
          if ( (z_param_out(1) == 0) .and. (z_param_out(2) == 0) ) then
            !
	      f2p = i20m1(mass2)
	    !   
          else  if ( (z_param_out(1) == 0) .and. (z_param_out(2) == 1) ) then
	    !
	       f2p(1) = 1._ki/2._ki
	       f2p(2) = 0._ki
	    !
	     if (rat_or_tot_par == 'rat') then
        !
               f2p(3) = 3._ki/4._ki
	       f2p(4) = 0._ki
         !
             else if (rat_or_tot_par == 'tot') then
        !
               f2p(3) = -(-3 + 2*real(z_log(mass2/mu2_scale_par,-1._ki)))/4._ki
	       f2p(4) = -aimag(z_log(mass2/mu2_scale_par,-1._ki))/2._ki
	    !
	     end if  ! end if rat or tot
        !
          else  if ( (z_param_out(1) == 0) .and. (z_param_out(2) == 2) ) then
	    !
	       f2p(1) = 1._ki/2._ki
	       f2p(2) = 0._ki
	    !
	     if (rat_or_tot_par == 'rat') then
        !
               f2p(3) = 1._ki/4._ki
	       f2p(4) = 0._ki
         !
             else if (rat_or_tot_par == 'tot') then
        !
               f2p(3) = -(-1 + 2*real(z_log(mass2/mu2_scale_par,-1._ki)))/4._ki
	       f2p(4) = -aimag(z_log(mass2/mu2_scale_par,-1._ki))/2._ki
	    !
             end if  ! end if rat or tot
        !
       else  if ( (z_param_out(1) == 1) .and. (z_param_out(2) == 1) ) then
	    !
	       f2p(1) = 1._ki/3._ki
	       f2p(2) = 0._ki
	    !
	     if (rat_or_tot_par == 'rat') then
        !
               f2p(3) = 11._ki/18._ki
	       f2p(4) = 0._ki
         !
             else if (rat_or_tot_par == 'tot') then
        !
               f2p(3) = (11 - 6*real(z_log(mass2/mu2_scale_par,-1._ki)))/18._ki
	       f2p(4) =  - aimag(z_log(mass2/mu2_scale_par,-1._ki))/3._ki
       	    !
             end if  ! end if rat or tot
	    !
          else  if ( (z_param_out(1) == 1) .and. (z_param_out(2) == 2) ) then
	    !
	       f2p(1) = 1._ki/6._ki
	       f2p(2) = 0._ki
	    !
	     if (rat_or_tot_par == 'rat') then
        !
               f2p(3) = 5._ki/36._ki
	       f2p(4) = 0._ki
         !
             else if (rat_or_tot_par == 'tot') then
        !
               f2p(3) = (5 - 6*real(z_log(mass2/mu2_scale_par,-1._ki)))/36._ki
	       f2p(4) =  - aimag(z_log(mass2/mu2_scale_par,-1._ki))/6._ki
       	    !
             end if  ! end if rat or tot
	    !
          else  if ( (z_param_out(1) == 2) .and. (z_param_out(2) == 1) ) then
	    !
	       f2p(1) = 1._ki/6._ki
	       f2p(2) = 0._ki
	    !
	     if (rat_or_tot_par == 'rat') then
        !
               f2p(3) = 5._ki/36._ki
	       f2p(4) = 0._ki
         !
             else if (rat_or_tot_par == 'tot') then
        !
               f2p(3) = (5 - 6*real(z_log(mass2/mu2_scale_par,-1._ki)))/36._ki
	       f2p(4) =  - aimag(z_log(mass2/mu2_scale_par,-1._ki))/6._ki
       	    !
             end if  ! end if rat or tot
	    !
          else  if ( (z_param_out(1) == 2) .and. (z_param_out(2) == 2) ) then
	    !
	       f2p(1) = 1._ki/3._ki
	       f2p(2) = 0._ki
	    !
	     if (rat_or_tot_par == 'rat') then
        !
               f2p(3) = 1._ki/9._ki
	       f2p(4) = 0._ki
         !
             else if (rat_or_tot_par == 'tot') then
        !
               f2p(3) = (1 - 3*real(z_log(mass2/mu2_scale_par,-1._ki)))/9._ki
	       f2p(4) =  - aimag(z_log(mass2/mu2_scale_par,-1._ki))/3._ki
       	    !
             end if  ! end if rat or tot
	    !
          end if  ! end test value of z1,z2
          !
	 ! ******************
	 ! ** eq. (A.10) ****
        else if (  (equal_real(s12,zero)).and.(.not.(equal_real(mass1,zero))).and. &
     &          (equal_real(diffm,zero)) ) then 
	! case p^2=0, m1 nonzero, m2=m1
        	 ! write(6,*) 'case (2c): s12 =0, m1 nonzero, m2=m1'
       !
          if ( (z_param_out(1) == 0) .and. (z_param_out(2) == 0) ) then
            !
	       f2p = f2p0m_1mi(mass1,0,0)
	    !   
          else  if ( (z_param_out(1) == 0) .and. (z_param_out(2) == 1) ) then
	    !
	       f2p = f2p0m_1mi(mass1,0,1)
	    !
          else  if ( (z_param_out(1) == 0) .and. (z_param_out(2) == 2) ) then
	    !
	       f2p = f2p0m_1mi(mass1,0,2)
	    !
          else  if ( (z_param_out(1) == 1) .and. (z_param_out(2) == 1) ) then
	    !
	       f2p = f2p0m_1mi(mass1,1,1)
	    !
          else  if ( (z_param_out(1) == 1) .and. (z_param_out(2) == 2) ) then
	    !
	       f2p = f2p0m_1mi(mass1,1,2)
	    !
          else  if ( (z_param_out(1) == 2) .and. (z_param_out(2) == 1) ) then
	    !
	       f2p = f2p0m_1mi(mass1,1,2)
	    !
          else  if ( (z_param_out(1) == 2) .and. (z_param_out(2) == 2) ) then
	    !
	       f2p = f2p0m_1mi(mass1,2,2)
	    !
          end if  ! end test value of z1,z2
        !    
	 ! ******************
	 ! ** eq. (A.8) ****
        else if (  (equal_real(s12,zero)).and.(.not.(equal_real(mass1,zero))).and. &
     &    (.not.(equal_real(mass2,zero))) .and.  .not.(equal_real(diffm,zero)) ) then 
	! case p^2=0, m1 nonzero, m2 nonzero, m2 NOT=m1
        	 ! write(6,*) 'case (2d): s12 =0, m1 and m2 nonzero, m2 not= m1 '
       !
          if ( (z_param_out(1) == 0) .and. (z_param_out(2) == 0) ) then
            !
	       f2p = f2p0m_m1m2(mass1,mass2,0,0)
	    !   
          else  if ( (z_param_out(1) == 0) .and. (z_param_out(2) == 1) ) then
	    !
	       f2p = f2p0m_m1m2(mass1,mass2,0,1)
	    !
          else  if ( (z_param_out(1) == 0) .and. (z_param_out(2) == 2) ) then
	    !
	       f2p = f2p0m_m1m2(mass1,mass2,0,2)
	    !
          else  if ( (z_param_out(1) == 1) .and. (z_param_out(2) == 1) ) then
	    !
	       f2p = f2p0m_m1m2(mass1,mass2,1,1)
	    !
          else  if ( (z_param_out(1) == 1) .and. (z_param_out(2) == 2) ) then
	    !
	       f2p = f2p0m_m1m2(mass1,mass2,1,2)
	    !
          else  if ( (z_param_out(1) == 2) .and. (z_param_out(2) == 1) ) then
	    !
	       f2p = f2p0m_m1m2(mass1,mass2,1,2)
	    !
          else  if ( (z_param_out(1) == 2) .and. (z_param_out(2) == 2) ) then
	    !
	       f2p = f2p0m_m1m2(mass1,mass2,2,2)
	    !
          end if  ! end test value of z1,z2
        !
	! ************ now case s12 nonzero **********************    
        else if (  (.not.(equal_real(s12,zero))).and.(equal_real(mass1,zero)).and. &
     &          (.not.(equal_real(mass2,zero))) ) then 
	! case  p^2 nonzero, m1=0, m2 nonzero
		 ! write(6,*) 'case (1a): s12 nonzero, m2 nonzero, m1 =0'
       !
	  i2sonem=i2sm1(s12,mass2)
         !
          if ( (z_param_out(1) == 0) .and. (z_param_out(2) == 0) ) then
            !
	      f2p = i2sm1(s12,mass2)
	    !
	   else  if ( (z_param_out(1) == 0) .and. (z_param_out(2) == 1) ) then
	    !
	       f2p(1) = 1._ki/2._ki
	       f2p(2) = 0._ki
	 !     
	     if (rat_or_tot_par == 'rat') then
        !
               f2p(3) = -( mass2 - (mass2 + s12)*i2sonem(3) )/(2*s12)
	       f2p(4) = 0._ki
         !
             else if (rat_or_tot_par == 'tot') then
        !
               f2p(3) = -(mass2 - (mass2 + s12)*i2sonem(3) - & 
     &                  mass2*real(z_log(mass2/mu2_scale_par,-1._ki)))/(2*s12)
	       f2p(4) = -(           - (mass2 + s12)*i2sonem(4) - & 
     &	                mass2*aimag(z_log(mass2/mu2_scale_par,-1._ki)))/(2*s12)
         !
             end if  ! end if rat or tot
	    !
          else  if ( (z_param_out(1) == 0) .and. (z_param_out(2) == 2) ) then
	    !
	       f2p(1) = 1._ki/2._ki
	       f2p(2) = 0._ki
	 !     
	     if (rat_or_tot_par == 'rat') then
        !
               f2p(3) = -((mass2 - s12)*i2sonem(3) - mass2)/(2*s12)
	       f2p(4) = 0._ki
         !
             else if (rat_or_tot_par == 'tot') then
        !
               f2p(3) = -((mass2 - s12)*i2sonem(3) + mass2*(-1 + real(z_log(mass2/mu2_scale_par,-1._ki))))/(2*s12)
	       f2p(4) = -((mass2 - s12)*i2sonem(4) + mass2*aimag(z_log(mass2/mu2_scale_par,-1._ki)))/(2*s12)
         !
             end if  ! end if rat or tot
    	    !
          else  if ( (z_param_out(1) == 1) .and. (z_param_out(2) == 1) ) then
	    !	       
	       f2p(1) = 1._ki/3._ki
	       f2p(2) = 0._ki
	 !     
	     if (rat_or_tot_par == 'rat') then
        !
               f2p(3) = (-6*mass2**2 - 9*mass2*s12 + s12**2 + &
     &                  6*(mass2**2 + mass2*s12 + s12**2)*i2sonem(3))/(18*s12**2)
	       f2p(4) = 0._ki
         !
             else if (rat_or_tot_par == 'tot') then
        !
               f2p(3) = (-6*mass2**2 - 9*mass2*s12 + s12**2 + & 
     &                  6*(mass2**2 + mass2*s12 + s12**2)*i2sonem(3) + & 
     &                  6*mass2*(mass2 + s12)*real(z_log(mass2/mu2_scale_par,-1._ki)))/(18*s12**2)
	       f2p(4) = ( 6*(mass2**2 + mass2*s12 + s12**2)*i2sonem(4) + & 
     &                  6*mass2*(mass2 + s12)*aimag(z_log(mass2/mu2_scale_par,-1._ki)))/(18*s12**2)
         !
             end if  ! end if rat or tot	       
	    !
          else  if ( (z_param_out(1) == 1) .and. (z_param_out(2) == 2) ) then
	    !
	       f2p(1) = 1._ki/6._ki
	       f2p(2) = 0._ki
	 !     
	     if (rat_or_tot_par == 'rat') then
        !
               f2p(3) = (6*mass2**2 - s12**2 +   &
     &                 3*(-2*mass2**2 + mass2*s12 + s12**2)*i2sonem(3) )/(18*s12**2)
	       f2p(4) = 0._ki
         !
             else if (rat_or_tot_par == 'tot') then
        !
               f2p(3) = (6*mass2**2 - s12**2 +   &
     &                 3*(-2*mass2**2 + mass2*s12 + s12**2)*i2sonem(3) + &
     &                 3*mass2*(-2*mass2 + s12)*real(z_log(mass2/mu2_scale_par,-1._ki)))/(18*s12**2)
	       f2p(4) = ( 3*(-2*mass2**2 + mass2*s12 + s12**2)*i2sonem(4) + &
     &                  3*mass2*(-2*mass2 + s12)*aimag(z_log(mass2/mu2_scale_par,-1._ki)))/(18*s12**2)
         !
             end if  ! end if rat or tot	        
	    !
          else  if ( (z_param_out(1) == 2) .and. (z_param_out(2) == 1) ) then
	    !
	       f2p(1) = 1._ki/6._ki
	       f2p(2) = 0._ki
	 !     
	     if (rat_or_tot_par == 'rat') then
        !
               f2p(3) = (6*mass2**2 - s12**2 +   &
     &                 3*(-2*mass2**2 + mass2*s12 + s12**2)*i2sonem(3) )/(18*s12**2)
	       f2p(4) = 0._ki
         !
             else if (rat_or_tot_par == 'tot') then
        !
               f2p(3) = (6*mass2**2 - s12**2 +   &
     &                 3*(-2*mass2**2 + mass2*s12 + s12**2)*i2sonem(3) + &
     &                 3*mass2*(-2*mass2 + s12)*real(z_log(mass2/mu2_scale_par,-1._ki)))/(18*s12**2)
	       f2p(4) = ( 3*(-2*mass2**2 + mass2*s12 + s12**2)*i2sonem(4) + &
     &                  3*mass2*(-2*mass2 + s12)*aimag(z_log(mass2/mu2_scale_par,-1._ki)))/(18*s12**2)
         !
             end if  ! end if rat or tot	        
	    !
          else  if ( (z_param_out(1) == 2) .and. (z_param_out(2) == 2) ) then
	    !
	       f2p(1) = 1._ki/3._ki
	       f2p(2) = 0._ki
	 !     
	     if (rat_or_tot_par == 'rat') then
        !
               f2p(3) = (-6*mass2**2 + 9*mass2*s12 + s12**2 + &
     &                  6*(mass2 - s12)**2*i2sonem(3) )/(18*s12**2)
	       f2p(4) = 0._ki
         !
             else if (rat_or_tot_par == 'tot') then
        !
               f2p(3) = (-6*mass2**2 + 9*mass2*s12 + s12**2 + &
     &                  6*(mass2 - s12)**2*i2sonem(3) +          &
     &                  6*mass2*(mass2 - 2*s12)*real(z_log(mass2/mu2_scale_par,-1._ki)))/(18*s12**2)
	       f2p(4) = ( 6*(mass2 - s12)**2*i2sonem(4) + &
     &                  6*mass2*(mass2 - 2*s12)*aimag(z_log(mass2/mu2_scale_par,-1._ki)))/(18*s12**2)
         !
             end if  ! end if rat or tot        
	    !
          end if  ! end test value of z1,z2
	 ! ******************
        else if (  (.not.(equal_real(s12,zero))).and.(.not.(equal_real(mass1,zero))).and. &
     &          (equal_real(mass2,zero)) ) then 
	! case p^2 nonzero, m1 nonzero, m2=0
	 ! write(6,*) 'case (1b): s12 nonzero, m1 nonzero, m2 =0'
          !
	  i2sonem=i2sm1(s12,mass1)
         !
          if ( (z_param_out(1) == 0) .and. (z_param_out(2) == 0) ) then
            !
	      f2p = i2sm1(s12,mass1)
! *	      write(6,*) 's12=',s12
! *	      write(6,*) 'mass1=',mass1
  	    !
	   else  if ( (z_param_out(1) == 0) .and. (z_param_out(2) == 1) ) then
	    !
	       f2p(1) = 1._ki/2._ki
	       f2p(2) = 0._ki
	 !     
	     if (rat_or_tot_par == 'rat') then
        !
               f2p(3) = -((mass1 - s12)*i2sonem(3) - mass1)/(2*s12)
	       f2p(4) = 0._ki
         !
             else if (rat_or_tot_par == 'tot') then
        !
               f2p(3) = -((mass1 - s12)*i2sonem(3) + &
     &                  mass1*(-1 + real(z_log(mass1/mu2_scale_par,-1._ki))))/(2*s12)
	       f2p(4) = -((mass1 - s12)*i2sonem(4) + &
     &	                mass1*aimag(z_log(mass1/mu2_scale_par,-1._ki)))/(2*s12)
         !
             end if  ! end if rat or tot     
	    !
          else  if ( (z_param_out(1) == 0) .and. (z_param_out(2) == 2) ) then
	    !
	       f2p(1) = 1._ki/2._ki
	       f2p(2) = 0._ki
	 !     
	     if (rat_or_tot_par == 'rat') then
        !
               f2p(3) = -( mass1 - (mass1 + s12)*i2sonem(3) )/(2*s12)
	       f2p(4) = 0._ki
         !
             else if (rat_or_tot_par == 'tot') then
        !
               f2p(3) = -(mass1 - (mass1 + s12)*i2sonem(3) - &
     &	               mass1*real(z_log(mass1/mu2_scale_par,-1._ki)))/(2*s12)
	       f2p(4) = -(       - (mass1 + s12)*i2sonem(4) - &
     &                 mass1*aimag(z_log(mass1/mu2_scale_par,-1._ki)))/(2*s12)
         !
             end if  ! end if rat or tot
    	    !
          else  if ( (z_param_out(1) == 1) .and. (z_param_out(2) == 1) ) then
	    !
	       f2p(1) = 1._ki/3._ki
	       f2p(2) = 0._ki
	 !     
	     if (rat_or_tot_par == 'rat') then
        !
               f2p(3) = (-6*mass1**2 + 9*mass1*s12 + s12**2 + & 
     &   	       6*(mass1 - s12)**2*i2sonem(3) )/(18*s12**2)
	       f2p(4) = 0._ki
         !
             else if (rat_or_tot_par == 'tot') then
        !
               f2p(3) = (-6*mass1**2 + 9*mass1*s12 + s12**2 + & 
     &	                6*(mass1 - s12)**2*i2sonem(3) +  &
     &                  6*mass1*(mass1 - 2*s12)*real(z_log(mass1/mu2_scale_par,-1._ki)))/(18*s12**2)
	       f2p(4) =( 6*(mass1 - s12)**2*i2sonem(4) + &
     &                 6*mass1*(mass1 - 2*s12)*aimag(z_log(mass1/mu2_scale_par,-1._ki)))/(18*s12**2)
         !
             end if  ! end if rat or tot        
     	    !
          else  if ( (z_param_out(1) == 1) .and. (z_param_out(2) == 2) ) then
	    !
	       f2p(1) = 1._ki/6._ki
	       f2p(2) = 0._ki
	 !     
	     if (rat_or_tot_par == 'rat') then
        !
               f2p(3) = (6*mass1**2 - s12**2 +   &
     &                 3*(-2*mass1**2 + mass1*s12 + s12**2)*i2sonem(3) )/(18*s12**2)
	       f2p(4) = 0._ki
         !
             else if (rat_or_tot_par == 'tot') then
        !
               f2p(3) = (6*mass1**2 - s12**2 +   &
     &                 3*(-2*mass1**2 + mass1*s12 + s12**2)*i2sonem(3) + &
     &                 3*mass1*(-2*mass1 + s12)*real(z_log(mass1/mu2_scale_par,-1._ki)))/(18*s12**2)
	       f2p(4) = ( 3*(-2*mass1**2 + mass1*s12 + s12**2)*i2sonem(4) + &
     &                  3*mass1*(-2*mass1 + s12)*aimag(z_log(mass1/mu2_scale_par,-1._ki)))/(18*s12**2)
         !
             end if  ! end if rat or tot	        
	    !
          else  if ( (z_param_out(1) == 2) .and. (z_param_out(2) == 1) ) then
	    !
	       f2p(1) = 1._ki/6._ki
	       f2p(2) = 0._ki
	 !     
	     if (rat_or_tot_par == 'rat') then
        !
               f2p(3) = (6*mass1**2 - s12**2 +   &
     &                 3*(-2*mass1**2 + mass1*s12 + s12**2)*i2sonem(3) )/(18*s12**2)
	       f2p(4) = 0._ki
         !
             else if (rat_or_tot_par == 'tot') then
        !
               f2p(3) = (6*mass1**2 - s12**2 +   &
     &                 3*(-2*mass1**2 + mass1*s12 + s12**2)*i2sonem(3) + &
     &                 3*mass1*(-2*mass1 + s12)*real(z_log(mass1/mu2_scale_par,-1._ki)))/(18*s12**2)
	       f2p(4) = ( 3*(-2*mass1**2 + mass1*s12 + s12**2)*i2sonem(4) + &
     &                  3*mass1*(-2*mass1 + s12)*aimag(z_log(mass1/mu2_scale_par,-1._ki)))/(18*s12**2)
         !
             end if  ! end if rat or tot	        
	    !
          else  if ( (z_param_out(1) == 2) .and. (z_param_out(2) == 2) ) then
	    !
	       f2p(1) = 1._ki/3._ki
	       f2p(2) = 0._ki
	 !     
	     if (rat_or_tot_par == 'rat') then
        !
               f2p(3) = (-6*mass1**2 - 9*mass1*s12 + s12**2 + &
     &                  6*(mass1**2 + mass1*s12 + s12**2)*i2sonem(3))/(18*s12**2)
	       f2p(4) = 0._ki
         !
             else if (rat_or_tot_par == 'tot') then
        !
               f2p(3) = (-6*mass1**2 - 9*mass1*s12 + s12**2 +  &
     &	                6*(mass1**2 + mass1*s12 + s12**2)*i2sonem(3) + &
     &                  6*mass1*(mass1 + s12)*real(z_log(mass1/mu2_scale_par,-1._ki)))/(18*s12**2)
	       f2p(4) = ( 6*(mass1**2 + mass1*s12 + s12**2)*i2sonem(4) + &
     &                  6*mass1*(mass1 + s12)*aimag(z_log(mass1/mu2_scale_par,-1._ki)))/(18*s12**2)
         !
             end if  ! end if rat or tot	       
       	    !
          end if  ! end test value of z1,z2
	  !
	 ! ******************
        else if (  (.not.(equal_real(s12,zero))).and.(.not.(equal_real(mass1,zero))).and. &
     &            (.not.(equal_real(mass2,zero))) ) then 
	! case p^2 nonzero, m1 nonzero, m2 nonzero, eq.(A.5)
	! includes case m1=m2
   	 ! write(6,*) 'cases (1c) and (1d): s12 nonzero, m1 and m2  nonzero'
       !
          if ( (z_param_out(1) == 0) .and. (z_param_out(2) == 0) ) then
            !
	       f2p = f2p_m1m2(s12,mass1,mass2,0,0)
	    !   
          else  if ( (z_param_out(1) == 0) .and. (z_param_out(2) == 1) ) then
	    !
	       f2p = f2p_m1m2(s12,mass1,mass2,0,1)
	    !
          else  if ( (z_param_out(1) == 0) .and. (z_param_out(2) == 2) ) then
	    !
	       f2p = f2p_m1m2(s12,mass1,mass2,0,2)
	    !
          else  if ( (z_param_out(1) == 1) .and. (z_param_out(2) == 1) ) then
	    !
	       f2p = f2p_m1m2(s12,mass1,mass2,1,1)
	    !
          else  if ( (z_param_out(1) == 1) .and. (z_param_out(2) == 2) ) then
	    !
	       f2p = f2p_m1m2(s12,mass1,mass2,1,2)
	    !
          else  if ( (z_param_out(1) == 2) .and. (z_param_out(2) == 1) ) then
	    !
	       f2p = f2p_m1m2(s12,mass1,mass2,1,2)
	    !
          else  if ( (z_param_out(1) == 2) .and. (z_param_out(2) == 2) ) then
	    !
	       f2p = f2p_m1m2(s12,mass1,mass2,2,2)
	    !
          end if  ! end test value of z1,z2
! ******************************************************************
	  !
	else 
	     !~ call print_error('something wrong with arguments of two-point function f2p')
	     !~ call print_error('s12=', arg_real=s12)
	     !~ call print_error('m1s=', arg_real=mass1)
	     !~ call print_error('m2s=', arg_real=mass2)
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'something wrong with arguments of two-point function f2p'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 's12= %f0'
          tab_erreur_par(2)%arg_real = s12
          tab_erreur_par(3)%a_imprimer = .true.
          tab_erreur_par(3)%chaine = 'm1s= %f0'
          tab_erreur_par(3)%arg_real = mass1
          tab_erreur_par(4)%a_imprimer = .true.
          tab_erreur_par(4)%chaine = 'm2s= %f0'
          tab_erreur_par(4)%arg_real = mass2
          call catch_exception(0)
        end if  ! end if s12,m1,m2 eq. zero
        !
      end if  ! end if ( minval(z_param_ini) == -1 )
      !
    end function f2p
    !
    !****f* src/integrals/two_point/generic_function_2p/f2p_np2
    ! NAME
    !
    !  Function f2p_np2
    !
    ! USAGE
    !
    !  real_dim4 = f2p_np2(s_mat,b_pro)
    !
    ! DESCRIPTION
    !
    !  This function computes the generic two point function in n+2 dimensions, 
    !  without Feynman parameters in the numerator
    !
    ! INPUTS
    !
    !  * s_mat -- a real (type ki) array of rank 2, the S matrix
    !  * b_pro -- an integer which represents the set of the four unpinched
    !    propagators
    !
    ! SIDE EFFECTS
    !
    !  No side effect
    !
    ! RETURN VALUE
    !
    !  It returns a real (type ki) array of rank 1 and shape 4
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    function f2p_np2(s_mat,b_pro)
      !
      real(ki), intent (in), dimension(:,:) :: s_mat
      integer, intent (in) :: b_pro
      real(ki),dimension(4) :: f2p_np2,i2sonem,i2sca
      !
      real(ki) :: arg1,arg2,arg3,s12,mass1,mass2,diffrm,cuts12,small
      real(ki) :: arg_log
      integer :: m1,m2
      integer, dimension(2) :: s
      !
    !    cuts12=1.d-15
       small=1.d-3
   !
      s = unpackb(b_pro,countb(b_pro))
      !
      m1 = s(1)
      m2 = s(2)
      !
        arg1 = s_mat(m1,m2)
        arg2 = s_mat(m1,m1)
        arg3 = s_mat(m2,m2)
        !
 ! internal masses	
        mass1 = -s_mat(m1,m1)/2._ki
        mass2 = -s_mat(m2,m2)/2._ki
        diffrm=sqrt(mass1)-sqrt(mass2)
      !
	s12=arg1+mass1+mass2
   !
      if ( (equal_real(s12,zero)) .and. (arg2 == zero) .and. (arg3 == zero) ) then
        !
	f2p_np2 = 0._ki
        !  call print_error('f2p_np2 with no scale is zero')
        tab_erreur_par(1)%a_imprimer = .true.
        tab_erreur_par(1)%chaine = 'In function f2p_np2:'
        tab_erreur_par(2)%a_imprimer = .true.
        tab_erreur_par(2)%chaine = 'no two point function with external mass = 0'
        call catch_exception(0)
        !
       else if ( (.not.(equal_real(s12,zero))) .and. (equal_real(mass1,zero)) .and. (equal_real(mass2,zero)) ) then 
	 ! massless case
        !
        arg_log = arg1/mu2_scale_par
        !
        f2p_np2(1) = 1._ki/6._ki
        f2p_np2(2) = 0._ki
        !
        if (rat_or_tot_par == 'tot') then
          !
          f2p_np2(3) = 4._ki/9._ki-real(z_log(-arg_log,-1._ki),ki)/6._ki
          f2p_np2(4) = -aimag(z_log(-arg_log,-1._ki))/6._ki
          !
        else if (rat_or_tot_par == 'rat') then
          !
          f2p_np2(3) = 4._ki/9._ki
          f2p_np2(4) = 0._ki
          !
        end if 
        !
        f2p_np2 = s12*f2p_np2
        !
   	 !*************** massive cases *******************************
	 ! added 07.08.09
	 ! ************************************************************
        else if (  (equal_real(s12,zero)) .and. (.not.(equal_real(mass1,zero))) .and. &
     &             (equal_real(mass2,zero)) ) then 
	! case p^2=0, m1 nonzero, m2=0
        !
	       f2p_np2(1) = -1._ki/2._ki
	       f2p_np2(2) = 0._ki
	 !     
	     if (rat_or_tot_par == 'rat') then
        !
               f2p_np2(3) = -3._ki/4._ki
	       f2p_np2(4) = 0._ki
         !
             else if (rat_or_tot_par == 'tot') then
        !
               f2p_np2(3) = -2._ki*(3._ki - 2*real(z_log(mass1/mu2_scale_par,-1._ki)))/8._ki
	       f2p_np2(4) = aimag(z_log(mass1/mu2_scale_par,-1._ki))/2._ki
         !
             end if  ! end if rat or tot
	!   
	        f2p_np2 = mass1*f2p_np2
        !
! ******************
        else if (  (equal_real(s12,zero)) .and. (equal_real(mass1,zero)) .and. &
     &          (.not.(equal_real(mass2,zero))) ) then 
	! case p^2=0, m2 nonzero, m1=0
        !
	       f2p_np2(1) = -1._ki/2._ki
	       f2p_np2(2) = 0._ki
	 !     
	     if (rat_or_tot_par == 'rat') then
        !
               f2p_np2(3) = -3._ki/4._ki
	       f2p_np2(4) = 0._ki
         !
             else if (rat_or_tot_par == 'tot') then
        !
               f2p_np2(3) = -2._ki*(3._ki - 2*real(z_log(mass2/mu2_scale_par,-1._ki)))/8._ki
	       f2p_np2(4) = aimag(z_log(mass2/mu2_scale_par,-1._ki))/2._ki
         !
             end if  ! end if rat or tot
	!   
		        f2p_np2 = mass2*f2p_np2
        !
 ! ******************
	 ! ** eq. (A.10), uses f2p_np2 = -2*B22 ****
        else if ( (equal_real(s12,zero)).and.(.not.(equal_real(mass1,zero))).and.(equal_real(diffrm,zero)) ) then 
	! case p^2=0, m1 nonzero, m2=m1
        !
	       f2p_np2 = -mass1*i20m1(mass1)
	!   
	! ******************
        else if ( (equal_real(s12,zero)).and.(.not.(equal_real(mass1,zero))).and.(.not.(equal_real(mass2,zero)))&
     &  	.and..not.(equal_real(diffrm,zero)) ) then 
	! case p^2=0, m1 nonzero, m2 nonzero, m1 not= m2
        !
	       f2p_np2(1) = -(mass1+mass2)/2._ki
	       f2p_np2(2) = 0._ki
	 !     
        if (abs(diffrm) > small ) then 
      ! 
	     if (rat_or_tot_par == 'rat') then
        !
               f2p_np2(3) = -2._ki*(3*mass1**2 - 3*mass2**2)/(8._ki*(mass1 - mass2))
	       f2p_np2(4) = 0._ki
         !
             else if (rat_or_tot_par == 'tot') then
        !
               f2p_np2(3) = -2._ki*(3*mass1**2 - 3*mass2**2 - & 
     &                        2*mass1**2*real(z_log(mass1/mu2_scale_par,-1._ki)) + &
     &                        2*mass2**2*real(z_log(mass2/mu2_scale_par,-1._ki)))/(8._ki*(mass1 - mass2))
	       f2p_np2(4) = -2._ki*( - 2*mass1**2*aimag(z_log(mass1/mu2_scale_par,-1._ki)) + &
     &                        2*mass2**2*aimag(z_log(mass2/mu2_scale_par,-1._ki)))/(8._ki*(mass1 - mass2))
         !
             end if  ! end if rat or tot
       !
      else  ! use expansion in (m2sq-m1sq) up to order 3
	!    
    !   write(6,*) 'using expanded expression for B22, diffrm=',diffrm
        if (rat_or_tot_par == 'rat') then
        !
         f2p_np2(3) = 2._ki*(mass1 + mass2)*(  &
     &	                     -19*mass1**2 + 8*mass1*mass2 - mass2**2 )/(48*mass1**2)
         f2p_np2(4) = 0._ki
        !
        else if (rat_or_tot_par == 'tot') then
        !
         f2p_np2(3) =2._ki*(mass1 + mass2)*(  &
     &	                     -19*mass1**2 + 8*mass1*mass2 - mass2**2 + &
     &                        12*mass1**2*real(z_log(mass1/mu2_scale_par,-1._ki)))/(48*mass1**2)
         f2p_np2(4) =2._ki*(mass1 + mass2)*(  &
     &                        12*mass1**2*aimag(z_log(mass1/mu2_scale_par,-1._ki)))/(48*mass1**2)
        !
        end if  ! end if rat or tot
	!    
      end if ! end if abs(diffrm) > small
   !	   
	!   
	! ******************
	!  s12 nonzero
	!********************
        else if ( (.not.(equal_real(s12,zero))).and.(equal_real(mass1,zero)).and.(.not.(equal_real(mass2,zero))) ) then 
	! case p^2 nonzero, m1=0, m2 nonzero
        !
	  i2sonem=i2sm1(s12,mass2)
        !
	       f2p_np2(1) = -2._ki*(3*mass2 - s12)/12._ki
	       f2p_np2(2) = 0._ki
	!
	      if (rat_or_tot_par == 'rat') then
        !
	       f2p_np2(3) = -2._ki*( (3*mass2**2 + 9*mass2*s12 - 2*s12**2 - & 
     &                                   3*(mass2 - s12)**2*i2sonem(3) )/(36*s12))
      !
	       f2p_np2(4) =  0._ki
         !
             else if (rat_or_tot_par == 'tot') then
        !
	       f2p_np2(3) =  -2._ki*((3*mass2**2 + 9*mass2*s12 - 2*s12**2 - &
     &	                   3*(mass2 - s12)**2*i2sonem(3) -  &
     &                     3*mass2*(mass2 + s12)*real(z_log(mass2/mu2_scale_par,-1._ki)))/(36*s12))
      !
	       f2p_np2(4) =  -2._ki*(( - 3*(mass2 - s12)**2*i2sonem(4) -  &
     &                     3*mass2*(mass2 + s12)*aimag(z_log(mass2/mu2_scale_par,-1._ki)))/(36*s12))
         !
            end if  ! end if rat or tot
	!   
	 ! ******************
        else if ( (.not.(equal_real(s12,zero))).and.(.not.(equal_real(mass1,zero))).and.(equal_real(mass2,zero)) ) then 
	! case p^2 nonzero, m1 nonzero, m2=0
        !
	   i2sonem=i2sm1(s12,mass1)
        !
               f2p_np2(1) = -2._ki*(3*mass1 - s12)/12
	       f2p_np2(2) = 0._ki
	!
	      if (rat_or_tot_par == 'rat') then
        !
	       f2p_np2(3) = -2._ki*( (3*mass1**2 + 9*mass1*s12 - 2*s12**2 - &
     &                                   3*(mass1 - s12)**2*i2sonem(3) )/(36*s12) )
      !
	       f2p_np2(4) =  0._ki
         !
             else if (rat_or_tot_par == 'tot') then
        !
	       f2p_np2(3) = -2._ki*( (3*mass1**2 + 9*mass1*s12 - 2*s12**2 - &
     &                                   3*(mass1 - s12)**2*i2sonem(3) -  &
     &                                   3*mass1*(mass1 + s12)*real(z_log(mass1/mu2_scale_par,-1._ki)))/(36*s12) )
      !
	       f2p_np2(4) = -2._ki*( ( - 3*(mass1 - s12)**2*i2sonem(4) - &
     &                                      3*mass1*(mass1 + s12)*aimag(z_log(mass1/mu2_scale_par,-1._ki)))/(36*s12) )
         !
            end if  ! end if rat or tot
	!   
	 ! ******************
        else if ( (.not.(equal_real(s12,zero))).and.(.not.(equal_real(mass1,zero))).and.  &
     &            (.not.(equal_real(mass2,zero))) ) then 
	! case p^2 nonzero, m1 nonzero, m2 nonzero, -2*B22 of eq.(A.5)
        !
	i2sca=i2sm1m2(s12,mass1,mass2)
	!
	       f2p_np2(1) = -2._ki*(3*mass1 + 3*mass2 - s12)/12._ki
	       f2p_np2(2) = 0._ki
	!
	      if (rat_or_tot_par == 'rat') then
        !
	       f2p_np2(3) = -2._ki*( (3*mass1**2 - 6*mass1*mass2 + 3*mass2**2 + &
     &                                   9*mass1*s12 + 9*mass2*s12 - 2*s12**2 - &
     &                                   3*(mass1**2 + (mass2 - s12)**2 - &
     &                                   2*mass1*(mass2 + s12))*i2sca(3) )/(36*s12) )
      !
	       f2p_np2(4) =  0._ki
         !
             else if (rat_or_tot_par == 'tot') then
        !
	       f2p_np2(3) = -2._ki*( (3*mass1**2 - 6*mass1*mass2 + 3*mass2**2 + & 
     &                                   9*mass1*s12 + 9*mass2*s12 - 2*s12**2 - &
     &                                   3*(mass1**2 + (mass2 - s12)**2 - &
     &                                   2*mass1*(mass2 + s12))*i2sca(3) - &
     &                                   3*mass1*(mass1 - mass2 + s12)*real(z_log(mass1/mu2_scale_par,-1._ki)) + &
     &                                   3*mass1*mass2*real(z_log(mass2/mu2_scale_par,-1._ki)) - &
     &                                   3*mass2**2*real(z_log(mass2/mu2_scale_par,-1._ki)) -  &
     &                                   3*mass2*s12*real(z_log(mass2/mu2_scale_par,-1._ki)))/(36*s12) )
      !
	       f2p_np2(4) = -2._ki*(-3*( mass1**2 + (mass2 - s12)**2  - &
     &                                    2*mass1*(mass2 + s12) )*i2sca(4) - &
     &                                    3*mass1*(mass1 - mass2 + s12)*aimag(z_log(mass1/mu2_scale_par,-1._ki)) + &
     &                                    3*mass1*mass2*aimag(z_log(mass2/mu2_scale_par,-1._ki)) -  &
     &                                    3*mass2**2*aimag(z_log(mass2/mu2_scale_par,-1._ki)) -  &
     &                                    3*mass2*s12*aimag(z_log(mass2/mu2_scale_par,-1._ki)) )/(36*s12) 
         !
            end if  ! end if rat or tot
!   
 ! ******************************************************************
      end if  ! end test if s12,m1,m2 zero
      !
    end function f2p_np2
    !
end module generic_function_2p
