! 
!****f* src/interface/gb0i
! NAME
!
!  Function gb0i
!
! USAGE
!
!  complex = gb0i(idt,s1,s2,s3,m1,m2,m3,mu2,eps_flag)
!
! DESCRIPTION
!
!  This function corresponds to the LoopTools b0i function.
!  The first argument is a character of length <= 4
!  There are two arguments more which are the renormalisation 
!  scale squared and a flag which selects the coefficient of 
!  the Laurent series in epsilon
!
! INPUTS
!
!  * idt -- a character of length <= 4, the type of form factors
!  * s1 -- a real (type ki), p1^2
!  * m1 -- a real (type ki), mass of propagator 2
!  * m2 -- a real (type ki), mass of propagator 1
!  * mu2 -- a real (type ki), renormalisation scale squared
!  * eps_flag -- an integer, a flag to select the coefficient in front the power of epsilon
!
! SIDE EFFECTS
!
!  No side effect
!
! RETURN VALUE
!
!  It returns a complex (type ki) corresponding
!  to the real part, imaginary part of the coefficient in front 1/epsilon^2 (eps_flag=-2),
!  the real part, imaginary part of the 1/epsilon term (eps_flag=-1) and the real part,
!  imaginary part of the constant term (eps_flag=0).
!
! USES
!
!  * precision (src/module/preci_double.f90)
!  * matrice_s, only: set_ref, s_mat, allocation_s, deallocation_s, init_invs (src/kinematic/matrice_s.f90)
!  * form_factor_type, only: form_factor (src/module/form_factor_type.f90)
!  * form_factor_2p (src/form_factor/form_factor_2p.f90)
!  * constante, only : b_null (src/module/constante.f90)
!  * tool_lt_to_golem, only : extract (src/interface/tool_lt_to_golem.f90)
!  * sortie_erreur, only : tab_erreur_par,catch_exception (src/module/sortie_erreur.f90)
!
!
! EXAMPLE
!
!
!
!*****
function gb0i(idt,s1,m1,m2,mu2,eps_flag)
  !
  use precision ! to get the type ki (for real and complex)
  use matrice_s, only: set_ref, s_mat, allocation_s, deallocation_s, init_invs
  use form_factor_type, only: form_factor
  use form_factor_2p ! module containing the three-point form factors (export all)
  use constante, only: b_null
  use tool_lt_to_golem, only : extract
  use sortie_erreur, only : tab_erreur_par,catch_exception
  implicit none
  !
  character (len=*), intent (in) :: idt
  real(ki), intent (in) :: s1,m1,m2,mu2
  integer, intent(in) :: eps_flag
  complex(ki) :: gb0i
  !
  integer, dimension(4) :: tab
  integer :: n1,n2
  integer, dimension(2) :: temp
  integer :: j1,j2
  type(form_factor) :: ff
  real(ki) :: lmu2
  character (len=4) :: idt_temp
  !
  call allocation_s(2)
  !
  set_ref = (/1,2/)
  !
  s_mat(1,1) = -2._ki*m2
  s_mat(1,2) = s1-m1-m2
  !
  s_mat(2,1) = s_mat(1,2)
  s_mat(2,2) = -2._ki*m1
  !
  call init_invs()
  !
  idt_temp = idt ! on complete la chaine entrante par des blancs
  call extract(idt_temp,tab)
  n1= count(tab/=-1)
  n2= count(tab/=-1 .and. tab/=0)
  !
  temp = -2
  temp(1:n1) = pack(tab,tab/=-1)
  !
  select case(n1)
    !
    case(1)
      !
      j1 = temp(1)
      !
      select case(n2)
      !
      case(0)
        !
        ff = a20(b_null)
        !
      case(1)
        !
        ff = a21(j1,b_null)
        !
      case default
        !
        tab_erreur_par(1)%a_imprimer = .true.
        tab_erreur_par(1)%chaine = 'In function gb0i (lt_to_golem.f90)'
        tab_erreur_par(2)%a_imprimer = .true.
        tab_erreur_par(2)%chaine = 'The value of n2 %d0'
        tab_erreur_par(2)%arg_int = n2
        tab_erreur_par(3)%a_imprimer = .true.
        tab_erreur_par(3)%chaine = 'is not compatible with n1 %d0'
        tab_erreur_par(3)%arg_int = n1
        call catch_exception(0)
        !
      end select
      !
    case(2)
      !
      j1 = temp(1)
      j2 = temp(2)
      !
      select case(n2)
      !
      case(0)
        !
        ff = b22(b_null)
        !
      case(2)
        !
        ff = a22(j1,j2,b_null)
        !
      case default
        !
        tab_erreur_par(1)%a_imprimer = .true.
        tab_erreur_par(1)%chaine = 'In function gb0i (lt_to_golem.f90)'
        tab_erreur_par(2)%a_imprimer = .true.
        tab_erreur_par(2)%chaine = 'The value of n2 %d0'
        tab_erreur_par(2)%arg_int = n2
        tab_erreur_par(3)%a_imprimer = .true.
        tab_erreur_par(3)%chaine = 'is not compatible with n1 %d0'
        tab_erreur_par(3)%arg_int = n1
        call catch_exception(0)
        !
      end select
      !
    case default
      !
      tab_erreur_par(1)%a_imprimer = .true.
      tab_erreur_par(1)%chaine = 'In function gb0i (lt_to_golem.f90)'
      tab_erreur_par(2)%a_imprimer = .true.
      tab_erreur_par(2)%chaine = 'The value of n1 is not correct %d0'
      tab_erreur_par(2)%arg_int = n1
      call catch_exception(0)
      !
  end select
  !
  lmu2 = log(mu2)
  !
  if (eps_flag == 0) then
    !
    gb0i = ff%c + lmu2*ff%b + lmu2**2*ff%a/2._ki
    !
  else if (eps_flag == -1) then
    !
    gb0i = ff%b + lmu2*ff%a 
    !
  else if (eps_flag == -2) then
    !
    gb0i = ff%a
    !
  end if
  !
  call deallocation_s()
  !  
end function gb0i
