! 
!****h* src/kinematic/inverse_matrice
! NAME
!
!  Module inverse_matrice
!
! USAGE
!
!  use inverse_matrice
!
! DESCRIPTION
!
! This module provides some routines and tools to inverse a n x n matrix. 
!
! OUTPUT
!
!  This module exports two routines:
!  * inverse -- to inverse a nXn matrix
!  * imprime_mat -- to print a nXn matrix
!
! USES
!
!  * precision (src/module/preci_double.f90)
!  * equal (src/module/equal.f90)
!
!*****
module inverse_matrice
  use precision
  use equal
  use parametre, only : accuracy_par
  use sortie_erreur, only : tab_erreur_par,catch_exception
  implicit none
  !
  private
  !
  public :: inverse, imprime_mat
  real(ki) :: glob_eps = 1.e-12_ki        ! valeur en de ca duquelle on
                                          ! passe au cas singulier
  contains
    !
    !****f* src/kinematic/inverse_matrice/inverse
    ! NAME
    !
    !  Subroutine inverse
    !
    ! USAGE
    !
    !  call inverse(mat,inv_mat,error,pinch1,pinch2)
    !
    ! DESCRIPTION
    !
    ! This routine first tries the Gauss method with partial pivoting strategy. 
    ! If the error returned is too large (greater than the global variable accuracy_par),
    ! then it switches to another method : the Greville method.
    ! In the case of the Gauss method, if some reduce matrices need to be inverted, a new matrix is built
    ! by removing the row(s) and column(s) pinch1,pinch2, then the inverse is computed and the result returned
    ! is a nXn matrix where the column(s) and row(s) pinch1, pinch2 are filled by 0, the other elements
    ! are those of the inverse computed. In the Greville method, the reduce matrix which is a nXn matrix
    ! where the column(s) and row(s) pinch1, pinch2 are filled by 0 is directly inverted.
    ! Note that the error is computed in the following way:
    ! first the matrix is rescaled : i. e. divided by the greatest (in absolute value) element
    ! then the inverse is computed and the two matrices abs(1 - A^(-1) A) and abs(1 - A A^(-1)) are computed
    ! the error is the greatest element of these two matrices.
    ! In the case of the Greville method, the Moore_Penrose conditions are also tested
    !
    ! INPUTS
    !
    !  * mat -- a real (type ki) array of rank 2
    !  * pinch1 -- an integer (optional), specified a pinch
    !  * pinch2 -- an integer (optional), specified a pinch
    !
    ! SIDE EFFECTS
    !
    !  No side effect
    !
    ! RETURN VALUE
    !
    !  * inv_mat -- a real (type ki) array of rank 2, same shape, the inverse 
    !               of the matrix mat
    !  * error -- a real (type ki), the estimation of the error of the numerical inversion
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    !
    subroutine inverse(mat,inv_mat,error,pinch1,pinch2)
      !
      real(ki), intent(in), dimension(:,:) :: mat
      real(ki), intent(out), dimension(size(mat,1),size(mat,1)) :: inv_mat
      real(ki), intent(out) :: error
      integer, optional :: pinch1,pinch2
      !
      real(ki), dimension(size(mat,1),size(mat,1)) :: norm_mat
      real(ki) :: plus_grand,g_error,o_error
      integer :: pin1,pin2
      !
      if (present(pinch1)) then ; pin1 = pinch1 ; else ; pin1 = -1 ; endif
      if (present(pinch2)) then ; pin2 = pinch2 ; else ; pin2 = -1 ; endif
      !
      g_error = 1._ki
      o_error = 1._ki
      !
      !
      ! First we rescale the matrix
      !
      plus_grand = maxval(array=abs(mat))
      norm_mat = mat/plus_grand
      !
      ! We first use the Gauss method
      !
      call inverse_rescue(norm_mat,inv_mat,o_error,pin1,pin2)
      !
      if (o_error >= accuracy_par) then
        !
        call inverse_greville(norm_mat,inv_mat,g_error)
        !
      end if
      !
      error = min(g_error,o_error)
      !
      if (error >= accuracy_par) then
        !
        tab_erreur_par(1)%a_imprimer = .true.
        tab_erreur_par(1)%chaine = 'alerte'
        tab_erreur_par(2)%a_imprimer = .true.
        tab_erreur_par(2)%chaine = 'the Greville method failed'
        tab_erreur_par(3)%a_imprimer = .true.
        tab_erreur_par(3)%chaine = 'the Gauss method failed too'
        tab_erreur_par(4)%a_imprimer = .true.
        tab_erreur_par(4)%chaine = 'the error returned %f0'
        tab_erreur_par(4)%arg_real = error
        call catch_exception(1)
        !
      end if
      !
      inv_mat = inv_mat/plus_grand
      !
    end subroutine inverse
    !
    !****if* src/kinematic/inverse_matrice/inverse_rescue
    ! NAME
    !
    !  Subroutine inverse_rescue
    !
    ! USAGE
    !
    !  call inverse_rescue(mat,inv_mat,error,pinch1,pinch2)
    !
    ! DESCRIPTION
    !
    ! The role of this routine is just to reduce the size the input 
    ! matrix mat if pinch1 and/or pinch2 is/are present
    !
    ! INPUTS
    !
    !  * mat -- a real (type ki) array of rank 2
    !  * pinch1 -- an integer (optional), specified a pinch
    !  * pinch2 -- an integer (optional), specified a pinch
    !
    ! SIDE EFFECTS
    !
    !  No side effect
    !
    ! RETURN VALUE
    !
    !  * inv_mat -- a real (type ki) array of rank 2, same shape, the inverse 
    !               of the matrix mat
    !  * error -- a real (type ki), the estimation of the error of the numerical inversion
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    !
    subroutine inverse_rescue(mat,inv_mat,error,pin1,pin2)
      !
      real(ki), intent(in), dimension(:,:) :: mat
      real(ki), intent(out), dimension(size(mat,1),size(mat,1)) :: inv_mat
      real(ki), intent(out) :: error
      integer, intent(in) :: pin1,pin2
      !
      integer :: i,n_dim
      logical, dimension(size(mat,1),size(mat,1)) :: masque
      real(ki), dimension(size(mat,1),size(mat,1)) :: f2
      real(ki), dimension(:,:), allocatable :: true_mat,true_inv_mat
      real(ki), dimension(:), allocatable :: true_vect
      integer, dimension(2) :: tab_pinch
      integer :: nb_pinch
      !
      tab_pinch = (/pin1,pin2/)
      nb_pinch = count(tab_pinch /= -1)
      masque = .true.
      !
      n_dim = size(mat,1)-nb_pinch
      !
      allocate(true_mat(n_dim,n_dim))
      allocate(true_inv_mat(n_dim,n_dim))
      allocate(true_vect(n_dim*n_dim))
      !
      select case (nb_pinch)
      !
      case(0)
        !
        true_mat = mat
        !
      case(1)
        !
        i = maxval(tab_pinch)
        masque(i,:) = .false.
        masque(:,i) = .false.
        true_vect = pack(mat,mask=masque)
        true_mat = reshape(source=true_vect,shape=(/n_dim,n_dim/))
        !
      case(2)
        !
        masque(pin1,:) = .false.
        masque(:,pin1) = .false.
        masque(pin2,:) = .false.
        masque(:,pin2) = .false.
        true_vect = pack(mat,mask=masque)
        true_mat = reshape(source=true_vect,shape=(/n_dim,n_dim/))
        !
      case default
        !
        !
      end select
      !
      call inverse_true(true_mat,true_inv_mat,error)
      !
      f2 = 0._ki
      true_vect = pack(true_inv_mat,.true.)
      inv_mat = unpack(true_vect,masque,f2)
      !
      deallocate(true_mat)
      deallocate(true_inv_mat)
      deallocate(true_vect)
      !
    end subroutine inverse_rescue
    !
    !****if* src/kinematic/inverse_matrice/inverse_true
    ! NAME
    !
    !  Subroutine inverse_true
    !
    ! USAGE
    !
    !  call inverse_true(mat,inv_mat,error)
    !
    ! DESCRIPTION
    !
    ! This routine uses a Gauss pivot method with partial pivoting to inverse M a nXn matrix.
    ! It returns an estimation of the error by computing the two matrices abs(1 - M^(-1) M) 
    ! and abs(1 - M M^(-1)). The error is the greatest element of these two matrices.
    !
    ! INPUTS
    !
    !  * mat -- a real (type ki) array of rank 2
    !
    ! SIDE EFFECTS
    !
    !  No side effect
    !
    ! RETURN VALUE
    !
    !  * inv_mat -- a real (type ki) array of rank 2, same shape, the inverse 
    !               of the matrix mat
    !  * error -- a real (type ki), the estimation of the error of the numerical inversion
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    !
    subroutine inverse_true(mat,inv_mat,error)
      !
      real(ki), intent(in), dimension(:,:) :: mat
      real(ki), intent(out), dimension(size(mat,1),size(mat,1)) :: inv_mat
      real(ki), intent(out) :: error
      !
      real(ki), dimension(size(mat,1),size(mat,1)) :: mat1,mat2,unit_mat
      real(ki), dimension(size(mat,1),size(mat,1)) :: p_mat,l_mat,u_mat
      real(ki), dimension(size(mat,1),size(mat,1)) :: inv_l_mat,inv_u_mat
      integer :: i,n_dim
      real(ki) :: max1,max2
      integer :: errorflag
      !
      n_dim = size(mat,1)               ! dimension de la matrice
      unit_mat = 0._ki
      !
      do i=1,n_dim
        !
        unit_mat(i,i) = 1._ki
        !
      end do
      !
      call lu_decomp(mat,p_mat,l_mat,u_mat)
      call inverse_triangular(l_mat,'inf',inv_l_mat)
      call inverse_triangular(u_mat,'sup',inv_u_mat)
      inv_mat = matmul(inv_u_mat,inv_l_mat)
      inv_mat = matmul(inv_mat,transpose(p_mat))
      !
      mat1 = matmul(inv_mat,mat)
      mat2 = matmul(mat,inv_mat)
      mat1 = abs(mat1-unit_mat)
      mat2 = abs(mat2-unit_mat)
      !
      max1 = maxval(mat1)
      max2 = maxval(mat2)
      !
      error = max(max1,max2)
      !
    end subroutine inverse_true
    !
    !****if* src/kinematic/inverse_matrice/inverse_greville
    ! NAME
    !
    !  Subroutine inverse_greville
    !
    ! USAGE
    !
    !  call inverse_greville(mat,inv_mat,error)
    !
    ! DESCRIPTION
    !
    !  This routine inverses a n x n matrix using the Greville method. 
    !  This method enables to invert singular matrix using the Moore-Penrose definition.
    !  One builds an iterative process, a rectengular matrix A is defined with the 
    !  first row of the n x n matrix, the the other rows are added step by step
    !  At each step, the pseudo inverse (Moore-Penrose) A' is built such that
    !  A A' = transpose(A A')
    !  A' A = transpose(A' A)
    !  A' A A' = A'
    !  A A' A = A
    !
    ! INPUTS
    !
    !  * mat -- a real (type ki) array of rank 2
    !
    ! SIDE EFFECTS
    !
    !  No side effect
    !
    ! RETURN VALUE
    !
    !  * inv_mat -- a real (type ki) array of rank 2, same shape, the inverse 
    !               of the matrix mat
    !  * error -- a real (type ki), the estimation of the error of the numerical inversion
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    ! On construit un processus iteratif : on definit une matrice A qui au 
    ! debut est construit en partant de la premiere colonne
    ! puis en ajoutant les colonnes une a une. 
    ! Pour chaque etape, on construit le pseudo inverse 
    ! (au sens de Moore-Penrose) A' tel que :
    ! A A' = transpose(A A')
    ! A' A = transpose(A' A)
    ! A' A A' = A'
    ! A A' A = A
    ! cette routine retourne une estimation de l'erreur
    !
    subroutine inverse_greville(mat,inv_mat,error)
      !
      real(ki), intent(in), dimension(:,:) :: mat
      real(ki), intent(out), dimension(size(mat,1),size(mat,1)) :: inv_mat
      real(ki), intent(out) :: error
      !
      real(ki), dimension(size(mat,1),1) :: a
      real(ki), dimension(1,size(mat,1)) :: at
      real(ki), dimension(:,:), allocatable :: ga,ga_prime
      real(ki), dimension(:,:), allocatable :: gab,gab_prime
      integer :: k,n_dim
      integer :: res
      real(ki) :: denom
      !
      n_dim = size(mat,1)               ! dimension de la matrice
      !
      ! premiere iteration
      !
      allocate(ga(n_dim,1),stat=res)
      allocate(ga_prime(1,n_dim),stat=res)
      !
      a(:,1) = mat(:,1)         ! premiere colonne de mat
      ga = a                         ! A est la 1ere colonne de mat
      at = transpose(a)
      denom = sum(matmul(at,a))         ! on utilise sum pour rendre scalaire
                                  ! matmul, sinon tableau (1,1)
      if (sqrt(denom) >= glob_eps) then
        !
        ga_prime = at/denom             ! cas standard A' pseudo-inverse de A
        !
      else
        !
        ga_prime = 0._ki                 ! si mat est singuliere
        !
      end if
      !
      ! autres iterations
      !
      do k=2,n_dim
        !
        allocate(gab(n_dim,k),stat=res)
        allocate(gab_prime(k,n_dim),stat=res)
        !
        a(:,1) = mat(:,k)
        call compt(n_dim,k-1,a,ga,ga_prime,gab,gab_prime)
        call verif(n_dim,k,gab,gab_prime,error)
        !
        deallocate(ga,stat=res)
        deallocate(ga_prime,stat=res)
        !
        allocate(ga(n_dim,k),stat=res)
        allocate(ga_prime(k,n_dim),stat=res)
        !
        ga = gab
        ga_prime = gab_prime
        !
        deallocate(gab,stat=res)
        deallocate(gab_prime,stat=res)
        !
      end do
      !
      inv_mat = ga_prime
      !
    end subroutine inverse_greville
    !
    !****if* src/kinematic/inverse_matrice/compt
    ! NAME
    !
    !  Subroutine compt
    !
    ! USAGE
    !
    !  call compt(n,k,a,ga,ga_prime,ga_plus,ga_prime_plus)
    !
    ! DESCRIPTION
    !
    !  This routine is used to iterate the inversion processus
    !
    ! INPUTS
    !
    !  * n -- an integer, the shape of the matrix to inverse
    !  * k -- an integer, number of rows
    !  * a -- a real (type ki) array of rank 2,  its shape is (n,1)
    !  * ga -- a real(type ki) array of rank 2, its shapes is (n,k) 
    !  * ga_prime -- a real(type ki) array of rank 2, its shapes is (k,n)
    !
    ! SIDE EFFECTS
    !
    !  No side effect
    !
    ! RETURN VALUE
    !
    !  * ga_plus -- a real(type ki) array of rank 2, its shape is (n,k+1)
    !  * ga_prime_plus -- a real(type ki) array of rank 2, its shape is (k+1,n)
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    ! cette routine itere le processus, elle recoit en argument les dimensions 
    ! de A at A' ainsi que A et A' et retourne A+ et A'+, A+ a une colonne de 
    ! plus que A et A'+ une ligne de plus que A'
    !
    subroutine compt(n,k,a,ga,ga_prime,ga_plus,ga_prime_plus)
      !
      integer, intent(in) :: n,k
      real(ki), intent(in), dimension(n,1) :: a
      real(ki), intent(in), dimension(n,k) :: ga
      real(ki), intent(in), dimension(k,n) :: ga_prime
      real(ki), intent(out), dimension(n,k+1) :: ga_plus
      real(ki), intent(out), dimension(k+1,n) :: ga_prime_plus
      !
      real(ki), dimension(k,1) :: d
      real(ki), dimension(1,k) :: dt
      real(ki), dimension(n,1) :: c
      real(ki), dimension(1,n) :: ct
      real(ki) :: denom
      !
      d = matmul(ga_prime,a)
      dt = transpose(d)
      c = a - matmul(ga,d)
      ct = transpose(c)                         ! cas standard
      denom = sum(matmul(ct,c))         ! cas standard
      !
      if (sqrt(denom) < glob_eps) then
        !
        ct = matmul(dt,ga_prime)        ! cas singulier
        denom = 1._ki + sum(matmul(dt,d))       ! cas singulier
        !
      end if
      !
      ga_prime_plus(1:k,:) = ga_prime - matmul(d,ct)/denom
      ga_prime_plus(k+1,:) = ct(1,:)/denom
      ga_plus(:,1:k) = ga
      ga_plus(:,k+1) = a(:,1)
      !
    end subroutine compt
    !
    !****if* src/kinematic/inverse_matrice/verif
    ! NAME
    !
    !  Subroutine verif
    !
    ! USAGE
    !
    !  call verif(n,k,ga,ga_prime,error)
    !
    ! DESCRIPTION
    !
    !  This routine verifies the Moor-Penrose conditions, it returns the 
    !  maximum error obtained
    !
    ! INPUTS
    !
    !  * n -- an integer, the shape of the matrix to inverse
    !  * k -- an integer, number of rows
    !  * ga -- a real(type ki) array of rank 2, its shapes is (n,k) 
    !  * ga_prime -- a real(type ki) array of rank 2, its shapes is (k,n)
    !
    ! SIDE EFFECTS
    !
    !  No side effect
    !
    ! RETURN VALUE
    !
    !  * error -- a real(type ki), the maximum error obtained to fulfill the 
    !             Moor-Penrose conditions
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    ! cette routine verifie les differentes conditions de Moor-Penrose, 
    ! elle retourne une erreur qui est le maximum des nombres trouves 
    ! au lieu de 0
    !
    subroutine verif(n,k,ga,ga_prime,error)
      !
      integer, intent(in) :: n,k
      real(ki), intent(in), dimension(n,k) :: ga
      real(ki), intent(in), dimension(k,n) :: ga_prime
      real(ki), intent (out) :: error
      !
      real(ki), dimension(n,n) :: mat1
      real(ki), dimension(k,k) :: mat2
      real(ki), dimension(n,k) :: mat3
      real(ki), dimension(k,n) :: mat4
      real(ki) :: max1,max2,max3,max4
      !
      mat1 = matmul(ga,ga_prime)
      mat2 = matmul(ga_prime,ga)
      mat3 = abs(matmul(mat1,ga)-ga)
      mat4 = abs(matmul(mat2,ga_prime)-ga_prime)
      !
      mat1 = abs(mat1-transpose(mat1))
      mat2 = abs(mat2-transpose(mat2))
      !
      max1 = maxval(mat1)
      max2 = maxval(mat2)
      max3 = maxval(mat3)
      max4 = maxval(mat4)
      !
      error = max(max1,max2,max3,max4)
      !
    end subroutine verif
    !
    !****f* src/kinematic/inverse_matrice/imprime_mat
    ! NAME
    !
    !  Subroutine imprime_mat
    !
    ! USAGE
    !
    !  call imprime_mat(mat)
    !
    ! DESCRIPTION
    !
    !  This routine prints a n x n matrix
    !
    ! INPUTS
    !
    !  * mat -- a real (type ki) array of rank 2
    !
    ! SIDE EFFECTS
    !
    !  No side effect
    !
    ! RETURN VALUE
    !
    !  No value returned
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    ! cette routine sert a imprimer une matrice carree
    !
    subroutine imprime_mat(mat)
      !
      real(ki), intent(in), dimension(:,:) :: mat
      !
      character(len=7*size(mat,1)+5*3) :: form 
      integer :: i
      integer, dimension(2) :: dim
      !
      dim = shape(mat)
      form = ""
      !
      do i=1,dim(2)-1
        !
        form = trim(form)//"e16.10,2x,"
        !
      end do
      !
      form = trim(form)//"e16.10"
      !
      do i=1,dim(1)
        !
        write (*,'(1x,"[",'//form//',"]")') mat(i,:)
        !
      end do
      !
    end subroutine imprime_mat
    !
    !****if* src/kinematic/inverse_matrice/inverse_triangular
    ! NAME
    !
    !  Subroutine inverse_triangular
    !
    ! USAGE
    !
    !  call inverse_triangular(mat,inf_or_sup,inv_mat)
    !
    ! DESCRIPTION
    !
    !  This routine inverses an upper or lower triangular matrix
    !  The program assumes that the matrix is lower triangular, if it is upper
    !  the program works with the transposed matrix
    !
    ! INPUTS
    !
    !  * mat -- a real (type ki) array of rank 2
    !  * inf_or_sup -- a string of 3 characters : inf or sup
    !
    ! SIDE EFFECTS
    !
    !  No side effect
    !
    ! RETURN VALUE
    !
    !  * inv_mat -- a real (type ki) array of rank 2, the inverse of the triangular matrix 
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    !
    subroutine inverse_triangular(mat,inf_or_sup,inv_mat)
      !
      real(ki), intent(in), dimension(:,:) :: mat
      character(len=3), intent(in) :: inf_or_sup
      real(ki), intent(out), dimension(size(mat,1),size(mat,1)) :: inv_mat
      !
      logical :: inversible,triangular_sup,triangular_inf
      integer :: n_dim,i,j,k
      real(ki) :: somme
      real(ki), dimension(size(mat,1),size(mat,1)) :: mat1
      !
      n_dim = size(mat,1)		! dimension de la matrice
      inversible = .true.
      triangular_inf = .false.
      triangular_sup = .false.
      inv_mat = 0._ki
      !
      if (inf_or_sup == 'inf') triangular_inf = .true.
      if (inf_or_sup == 'sup') triangular_sup = .true.
      !
      do i=1,n_dim
        !
        inversible = inversible .and. .not.(equal_real(mat(i,i),0._ki))
        !
      end do
      !
      if ( (inversible) .and. (triangular_inf .or. triangular_sup) ) then
        !
        if (triangular_inf) then
          !
          mat1 = mat
          !
        else
          !
          mat1 = transpose(mat)
          !
        end if
        !
        do i=1,n_dim
          !
          do j=1,i
            !
            if (j == i) then
              !
              inv_mat(i,j) = 1._ki/mat1(i,i)
              !
            else
              !
              somme = 0._ki
              !
              do k=1,i-1
                !
                somme = somme + mat1(i,k)*inv_mat(k,j)
                !
              end do
              !
              inv_mat(i,j) = -somme/mat1(i,i)
              !
            end if
            !
          end do
          !
        end do
        !
        if (triangular_sup) inv_mat = transpose(inv_mat)
        !
      else
        !
        !~ if (.not.(inversible)) write(*,*) 'matrice pas inversible'
        if (.not.(inversible)) then
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'alerte, internal error'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'In the LU decomposition'
          tab_erreur_par(3)%a_imprimer = .true.
          tab_erreur_par(3)%chaine = 'One triangular matrix is not invertible'
          call catch_exception(1)
        end if
        !~ if (triangular_inf .and. triangular_sup) write(*,*) 'matrice pas diagonal'
        if (triangular_inf .and. triangular_sup) then
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'alerte, internal error'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'In the LU decomposition'
          tab_erreur_par(3)%a_imprimer = .true.
          tab_erreur_par(3)%chaine = 'One matrix is not diagonal'
          call catch_exception(1)
        end if
        !
      end if
      !
    end subroutine inverse_triangular
    !
    !****if* src/kinematic/inverse_matrice/inverse_diagonal
    ! NAME
    !
    !  Subroutine inverse_diagonal
    !
    ! USAGE
    !
    !  call inverse_diagonal(mat,inv_mat)
    !
    ! DESCRIPTION
    !
    !  This routine inverses a diagonal matrix
    !
    ! INPUTS
    !
    !  * mat -- a real (type ki) array of rank 2
    !
    ! SIDE EFFECTS
    !
    !  No side effect
    !
    ! RETURN VALUE
    !
    !  * inv_mat -- a real (type ki) array of rank 2, the inverse of the triangular matrix 
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    !
    subroutine inverse_diagonal(mat,inv_mat)
      !
      real(ki), intent(in), dimension(:,:) :: mat
      real(ki), intent(out), dimension(size(mat,1),size(mat,1)) :: inv_mat
      !
      logical :: inversible
      integer :: n_dim,i
      !
      n_dim = size(mat,1)		! dimension de la matrice
      inversible = .true.
      inv_mat = 0._ki
      !
      do i=1,n_dim
        !
        inversible = inversible .and. (mat(i,i) /= 0._ki)
        !
      end do
      !
      if (inversible) then
        !
        do i=1,n_dim
          !
          inv_mat(i,i) = 1._ki/mat(i,i)
          !
        end do
        !
      else
        !
        !~ write(*,*) 'matrice pas inversible'
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'alerte, internal error'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'In the LU decomposition'
          tab_erreur_par(3)%a_imprimer = .true.
          tab_erreur_par(3)%chaine = 'One diagonal matrix is not invertible'
          call catch_exception(1)
        !
      end if
      !
    end subroutine inverse_diagonal
    !
    !
    !****if* src/kinematic/inverse_matrice/lu_decomp
    ! NAME
    !
    !  Subroutine lu_decomp
    !
    ! USAGE
    !
    !  call lu_decomp(mat,p_mat,l_mat,u_mat)
    !
    ! DESCRIPTION
    !
    !  This routine writes a symetric matrix as P L U where P is a
    !  permutation matrix, L is a lower triangular matrix and  
    !  U is an upper triangular matrix
    !
    ! INPUTS
    !
    !  * mat -- a real (type ki) array of rank 2
    !
    ! SIDE EFFECTS
    !
    !  No side effect
    !
    ! RETURN VALUE
    !
    !  * p_mat -- a real (type ki) array of rank 2, the permutation matrix 
    !  * l_mat -- a real (type ki) array of rank 2, the lower triangular matrix 
    !  * u_mat -- a real (type ki) array of rank 2, the upper triangular matrix 
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    !
    subroutine lu_decomp(mat,p_mat,l_mat,u_mat)
      !
      real(ki), intent(in), dimension(:,:) :: mat
      real(ki), intent(out), dimension(size(mat,1),size(mat,1)) :: l_mat,u_mat,p_mat
      !
      integer :: n_dim,i,j,k,l,p
      real(ki) :: temp1,temp2,res
      integer, dimension(1) :: loc_m
      real(ki), dimension(size(mat,1),size(mat,1)) :: mat_b
      real(ki), dimension(size(mat,1),size(mat,1)) :: temp_mat,id_mat
      real(ki), dimension(size(mat,1)) :: temp_vec
      !
      n_dim = size(mat,1)		! dimension de la matrice
      u_mat = 0._ki
      id_mat = 0._ki
      !
      do i=1,n_dim
        !
        id_mat(i,i) = 1._ki
        !
      end do
      !
      mat_b = mat
      l_mat = 0._ki
      p_mat = id_mat
      !
      do k=1,n_dim-1
        !
        ! plus grand element de la colonne k entre les lignes k et n_dim
        !
        loc_m = maxloc(abs(mat_b(k:n_dim,k))) + k - 1 
        !
        ! si ce plus grand element n'est pas sur la ligne k ou bien on ne 
        ! traite pas la derniere ligne, on permute les lignes.
        ! p_mat garde trace de la permutation
        !
        if ( (loc_m(1) /= k) .and.(k /= n_dim) ) then
          !
          temp_vec = mat_b(k,:)
          mat_b(k,:) = mat_b(loc_m(1),:)
          mat_b(loc_m(1),:) = temp_vec
          !
          temp_vec = l_mat(k,:)
          l_mat(k,:) = l_mat(loc_m(1),:)
          l_mat(loc_m(1),:) = temp_vec
          !
          temp_mat = id_mat
          temp_mat(k,:) = id_mat(loc_m(1),:)
          temp_mat(loc_m(1),:) = id_mat(k,:)
          p_mat = matmul(p_mat,temp_mat)
          !
        end if
        !
        l_mat(k,k) = 1._ki
        l_mat(k+1:n_dim,k) = mat_b(k+1:n_dim,k)/mat_b(k,k)
        !
        do i=k+1,n_dim
          !
          mat_b(i,1:k) = 0._ki
          !
          do j=k+1,n_dim
            !
            mat_b(i,j) = mat_b(i,j) - l_mat(i,k)*mat_b(k,j)
            !
          end do
          !
        end do
        !
      end do
      !
      l_mat(n_dim,n_dim) = 1._ki
      u_mat = mat_b
      !
    end subroutine lu_decomp
    !
end module inverse_matrice
