! 
!****h* src/kinematic/matrice_s
! NAME
!
!  Module matrice_s
!
! USAGE
!
!  use matrice_s
!
! DESCRIPTION
!
!  This module is used : to reserve some memory in order to pass the S matrix, its
!  shape, the set of propagator labels; to compute the inverse 
!  of S matrix and the related quantities : the b's and sumb, also for the 
!  reduce matrix (up to 4 X 4). The S matrix is allocated here and also its dimension
!  and it returns the result through the three functions inv_s, b and sumb.
!
! OUTPUT
!
!  This module exports three variable:
!  * dim_s -- an integer, the shape of the initial S matrix
!  * set_ref -- an integer array, the set of initial propagators
!  * s_mat -- a real (type ki) array of rank 2, the S matrix
!  and also seven functions:
!  * allocation_s -- to allocate the required memory
!  * deallocation_s -- to deallocate the used memory
!  * init_invs -- to fill all the array for the inverse of the S matrix
!                 and the inverse of the reduce S matrix
!  * inv_s -- it contains the inverse of the S matrix
!  * hj -- it contains H matrix (pseudo-inverse of G)
!  * b -- it contains the b coefficients
!  * sumb -- it contains the B coeficient
!  * norma_sumb -- it contains the normalised B coefficient
!
!  Only dim_s take a value in this module, not the other variables
!
! USES
!
!  * precision (src/module/preci_double.f90)
!  * sortie_erreur (src/module/sortie_erreur.f90)
!  * inverse_matrice (src/kinematic/inverse_matrice.f90)
!  * tri_croissant (src/module/tri_croissant.f90)
!  * array (src/module/array.f90)
!
!*****
module matrice_s
  !
  use precision
  use sortie_erreur
  use inverse_matrice
  use tri_croissant
  use array
  implicit none
  !
  private
  !
  integer :: dim_s
  integer, dimension(:), allocatable :: set_ref
  integer :: b_ref
  real(ki), dimension(:,:), allocatable :: s_mat
  !
  public :: dim_s,set_ref,b_ref,s_mat
  !
  real(ki), dimension(:,:), allocatable :: invs6
  real(ki), dimension(:,:,:), allocatable :: hjj
  real(ki), dimension(:), allocatable :: b6
  real(ki) :: sumb6
  real(ki), dimension(:,:,:), allocatable :: invs5
  real(ki), dimension(:,:), allocatable :: b5
  real(ki), dimension(:), allocatable :: sumb5
  real(ki), dimension(:,:,:,:), allocatable :: invs4
  real(ki), dimension(:,:,:), allocatable :: b4
  real(ki), dimension(:,:), allocatable :: sumb4
  real(ki), dimension(:,:), allocatable :: norma_sumb4
  integer :: err
  !
  public :: allocation_s,deallocation_s,init_invs,inv_s,hj,b,sumb,norma_sumb
  !
  contains
    !
    !****f* src/kinematic/matrice_s/allocation_s
    ! NAME
    !
    !  Subroutine allocation_s
    !
    ! USAGE
    !
    !  call allocation_s(dim)
    !
    ! DESCRIPTION
    !
    !  This subroutine reserves the memory for several objects
    !  dim_s is an integer, invs6 a real(type ki) array of rank 2
    !  hjj a real (type ki) array of rank 3, b6 a real (type ki) array of rank 1, 
    !  sumb6 a real (type ki), invs5 a real (type ki) array of rank 3, 
    !  b5 a real (type ki) array of rank 2, sumb5 a real (type ki) array of rank 1, 
    !  invs4 a real (type ki) array of rank 4, b4 a real (type ki) array of rank 3, 
    !  sumb4 a real (type ki) array of rank 2
    !  norma_sumb4 a real (type ki) array of rank 2
    !
    ! INPUTS
    !
    !  * dim -- an integer, the maximal number of external legs
    !
    ! SIDE EFFECTS
    !
    !  This routine modify the value of the variable dim_s
    !  It initialises invs6, hjj, b6, sumb6, invs5, b5, sumb5, 
    !  invs4, b4, sumb4, norma_sumb4 to zero
    !
    ! RETURN VALUE
    !
    !  No return value
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    subroutine allocation_s(dim)
      !
      integer, intent(in) :: dim
      integer :: err
      !
      dim_s = dim
      allocate(s_mat(dim_s,dim_s),stat=err)
      !
      if (err /= 0) then
        !
        tab_erreur_par(1)%a_imprimer = .true.
        tab_erreur_par(1)%chaine = 'In subroutine allocation_s'
        tab_erreur_par(2)%a_imprimer = .true.
        tab_erreur_par(2)%chaine = 'cannot allocate memory for s_mat'
        call catch_exception(0)
        !
      end if
      !
      allocate(set_ref(dim_s),stat=err)
      !
      if (err /= 0) then
        !
        tab_erreur_par(1)%a_imprimer = .true.
        tab_erreur_par(1)%chaine = 'In subroutine allocation_s'
        tab_erreur_par(2)%a_imprimer = .true.
        tab_erreur_par(2)%chaine = 'cannot allocate memory for set_ref'
        call catch_exception(0)
        !
      end if
      !
      select case(dim)
      !
      case(6)
        !
        allocate(invs6(6,6),hjj(6,6,6),b6(6),stat=err)
        !
        if (err /= 0) then
          !
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'In subroutine allocation_s case(6)'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'cannot allocate memory for invs6 or hjj or b6'
          call catch_exception(0)
          !
        end if
        !
        allocate(invs5(6,6,6),b5(6,6),sumb5(6),stat=err)
        !
        if (err /= 0) then
          !
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'In subroutine allocation_s case(6)'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'cannot allocate memory for invs5 or b5 or sumb5'
          call catch_exception(0)
          !
        end if
        !
        allocate(invs4(6,6,6,6),b4(6,6,6),sumb4(6,6),norma_sumb4(6,6),stat=err)
        !
        if (err /= 0) then
          !
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'In subroutine allocation_s case(6)'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'cannot allocate memory for invs4 or b4 or sumb4 or norma_sumb4'
          call catch_exception(0)
          !
        end if
        !
      case(5)
        !
        allocate(invs6(1,1),hjj(1,1,1),b6(1),stat=err)
        !
        if (err /= 0) then
          !
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'In subroutine allocation_s case(5)'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'cannot allocate memory for invs6 or hjj or b6'
          call catch_exception(0)
          !
        end if
        !
        allocate(invs5(5,5,1),b5(5,1),sumb5(1),stat=err)
        !
        if (err /= 0) then
          !
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'In subroutine allocation_s case(5)'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'cannot allocate memory for invs5 or b5 or sumb5'
          call catch_exception(0)
          !
        end if
        !
        allocate(invs4(5,5,1,5),b4(5,1,5),sumb4(1,5),norma_sumb4(1,5),stat=err)
        !
        if (err /= 0) then
          !
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'In subroutine allocation_s case(5)'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'cannot allocate memory for invs4 or b4 or sumb4 or norma_sumb4'
          call catch_exception(0)
          !
        end if
        !
      case(4)
        !
        allocate(invs6(1,1),hjj(1,1,1),b6(1),stat=err)
        !
        if (err /= 0) then
          !
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'In subroutine allocation_s case(4)'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'cannot allocate memory for invs6 or hjj or b6'
          call catch_exception(0)
          !
        end if
        !
        allocate(invs5(1,1,1),b5(1,1),sumb5(1),stat=err)
        !
        if (err /= 0) then
          !
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'In subroutine allocation_s case(4)'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'cannot allocate memory for invs5 or b5 or sumb5'
          call catch_exception(0)
          !
        end if
        !
        allocate(invs4(4,4,1,1),b4(4,1,1),sumb4(1,1),norma_sumb4(1,1),stat=err)
        !
        if (err /= 0) then
          !
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'In subroutine allocation_s case(4)'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'cannot allocate memory for invs4 or b4 or sumb4 or norma_sumb4'
          call catch_exception(0)
          !
        end if
        !
      case(2,3)
        ! in these cases invsn, hjj, bn, sumbn are of no use, one allocates the
        ! minimum
        !
        allocate(invs6(1,1),hjj(1,1,1),b6(1),stat=err)
        !
        if (err /= 0) then
          !
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'In subroutine allocation_s case(3)'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'cannot allocate memory for invs6 or hjj or b6'
          call catch_exception(0)
          !
        end if
        !
        allocate(invs5(1,1,1),b5(1,1),sumb5(1),stat=err)
        !
        if (err /= 0) then
          !
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'In subroutine allocation_s case(3)'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'cannot allocate memory for invs5 or b5 or sumb5'
          call catch_exception(0)
          !
        end if
        !
        allocate(invs4(1,1,1,1),b4(1,1,1),sumb4(1,1),norma_sumb4(1,1),stat=err)
        !
        if (err /= 0) then
          !
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'In subroutine allocation_s case(3)'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'cannot allocate memory for invs4 or b4 or sumb4 or norma_sumb4'
          call catch_exception(0)
          !
        end if
        !
      case default
        !
        tab_erreur_par(1)%a_imprimer = .true.
        tab_erreur_par(1)%chaine = 'In subroutine allocation_s'
        tab_erreur_par(2)%a_imprimer = .true.
        tab_erreur_par(2)%chaine = 'the argument dim has the value: %d0'
        tab_erreur_par(2)%arg_int = dim
        tab_erreur_par(3)%a_imprimer = .true.
        tab_erreur_par(3)%chaine = 'which is not correct, must be 2 <= dim <= 6'
        call catch_exception(0)
        !
      end select
      !
      invs6 = 0._ki
      b6 = 0._ki
      sumb6 = 0._ki
      hjj = 0._ki
      !
      invs5 = 0._ki
      b5 = 0._ki
      sumb5 = 0._ki
      !
      invs4 = 0._ki
      b4 = 0._ki
      sumb4 = 0._ki
      norma_sumb4 = 0._ki
      !
    end subroutine allocation_s
    !
    !
    !****f* src/kinematic/matrice_s/deallocation_s
    ! NAME
    !
    !  Subroutine deallocation_s
    !
    ! USAGE
    !
    !  call deallocation_s()
    !
    ! DESCRIPTION
    !
    !  This subroutine deallocates the memory reserved by the preceeding 
    !  subroutine
    !
    ! INPUTS
    !
    !  No input
    !
    ! SIDE EFFECTS
    !
    !  This routine destroies all the variables initialised in the 
    !  preceeding subroutine
    !
    ! RETURN VALUE
    !
    !  No return value
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    subroutine deallocation_s()
      !
      integer :: err
      !
      deallocate(s_mat,stat=err)
      !
      if (err /= 0) then
        !
        tab_erreur_par(1)%a_imprimer = .true.
        tab_erreur_par(1)%chaine = 'In subroutine deallocation_s'
        tab_erreur_par(2)%a_imprimer = .true.
        tab_erreur_par(2)%chaine = 'cannot deallocate memory for s_mat'
        call catch_exception(0)
      end if
      !
      deallocate(set_ref,stat=err)
      !
      if (err /= 0) then
        tab_erreur_par(1)%a_imprimer = .true.
        tab_erreur_par(1)%chaine = 'In subroutine deallocation_s'
        tab_erreur_par(2)%a_imprimer = .true.
        tab_erreur_par(2)%chaine = 'cannot deallocate memory for s_ref'
        call catch_exception(0)
      end if
      !
      deallocate(invs6,hjj,b6,stat=err)
      !
      if (err /= 0) then
        tab_erreur_par(1)%a_imprimer = .true.
        tab_erreur_par(1)%chaine = 'In subroutine deallocation_s'
        tab_erreur_par(2)%a_imprimer = .true.
        tab_erreur_par(2)%chaine = 'cannot deallocate memory for invs6 or hjj or b6'
        call catch_exception(0)
      end if
      !
      deallocate(invs5,b5,sumb5,stat=err)
      !
      if (err /= 0) then
        tab_erreur_par(1)%a_imprimer = .true.
        tab_erreur_par(1)%chaine = 'In subroutine deallocation_s'
        tab_erreur_par(2)%a_imprimer = .true.
        tab_erreur_par(2)%chaine = 'cannot deallocate memory for invs5 or b5 or sumb5'
        call catch_exception(0)
      end if
      !
      deallocate(invs4,b4,sumb4,norma_sumb4,stat=err)
      !
      if (err /= 0) then
        tab_erreur_par(1)%a_imprimer = .true.
        tab_erreur_par(1)%chaine = 'In subroutine deallocation_s'
        tab_erreur_par(2)%a_imprimer = .true.
        tab_erreur_par(2)%chaine = 'cannot deallocate memory for invs4 or b4 or sumb4 or norma_sumb4'
        call catch_exception(0)
      end if
      !
    end subroutine deallocation_s
    !
    !
    !****f* src/kinematic/inversion/init_invs
    ! NAME
    !
    !  Subroutine init_invs
    !
    ! USAGE
    !
    !  call init_invs()
    !
    ! DESCRIPTION
    !
    ! For a 6 x 6 S matrix, this routine fills the arrays: 
    ! invs6, hjj, b6, sumb6, invs5, b5, sumb5, invs4, b4, sumb4, norma_sumb4
    ! For a 5 x 5 S matrix, this routine fills the arrays: 
    ! invs5, b5, sumb5, invs4, b4, sumb4, norma_sumb4
    ! For a 4 x 4 S matrix, this routine fills the arrays: 
    ! invs4, b4, sumb4, norma_sumb4
    ! One can print a typycal error due to the numerical inversion
    !
    ! INPUTS
    !
    !  No input
    !
    ! SIDE EFFECTS
    !
    !  This routine modify the value invs6, hjj, b6, 
    !  sumb6, invs5, b5, sumb5, invs4, b4, sumb4, norma_sumb4
    !
    ! RETURN VALUE
    !
    !  No retrurn value
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    subroutine init_invs()
      !
      integer :: i1,i2,i
      real(ki), dimension(:,:), allocatable :: temp_mat,temp1_mat
      real(ki) :: error6,error5,error4,error
      real(ki) :: plus_grand
      !
      plus_grand = maxval(array=abs(s_mat))
      b_ref = packb(set_ref)
      !
      select case(dim_s)
      !
      case(6)
        !
        allocate(temp_mat(6,6),temp1_mat(6,6),stat=err)
        !
        if (err /= 0) then
          !
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'In subroutine init_invs case(6)'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'cannot allocate memory for temp_mat or temp1_mat'
          call catch_exception(0)
          !
        end if
        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        !!!! 6-point functions
        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        origine_inv_info_par = '6x6 matrix'
        call inverse(s_mat,invs6,error6)
        !
        b6(:) = sum(invs6,dim=1)
        sumb6 = sum(b6)
        !
        do i=1,6
          !
          do i1=1,6
            !
            do i2=1,6
              !
              hjj(i1,i2,i) = -2._ki*(  invs6(i1,i2) &
                                 - invs6(i,i1)*b6(i2)/b6(i) &
                                 - invs6(i,i2)*b6(i1)/b6(i) &
                                 + invs6(i,i)*b6(i1)*b6(i2) &
                                 /b6(i)**2 )
              !
            end do
            !
          end do
          !
        end do
        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        !!!! 5-point functions
        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        error5 = 0._ki
        !
        do i1=1,6
          !
          call put_to_zero(i1,s_mat,temp_mat)
          origine_inv_info_par = '5x5 matrix, pinch '//achar(i1+48)
          !~ call inverse(temp_mat,invs5(:,:,i1),error)
          call inverse(temp_mat,invs5(:,:,i1),error,i1)
          !
          b5(:,i1) = sum(invs5(:,:,i1),dim=1)
          sumb5(i1) = sum(b5(:,i1))
          error5 = max(error,error5)
          !
        end do
        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        !!!! 4-point functions
        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        error4 = 0._ki
        !
        do i1=1,6
          !
          call put_to_zero(i1,s_mat,temp_mat)
          !
          do i2=i1,6
            !
            if (i2 /= i1) then
              !
              call put_to_zero(i2,temp_mat,temp1_mat)
              origine_inv_info_par = '4x4 matrix, pinch '//achar(i1+48)//' '//achar(i2+48)
              !~ call inverse(temp1_mat,invs4(:,:,i1,i2),error)
              call inverse(temp1_mat,invs4(:,:,i1,i2),error,i1,i2)
              !
              b4(:,i1,i2) = sum(invs4(:,:,i1,i2),dim=1)
              sumb4(i1,i2) = sum(b4(:,i1,i2))
              norma_sumb4(i1,i2) = sumb4(i1,i2)*plus_grand
              error4 = max(error,error4)
              !
            end if
            !
          end do
          !
        end do
        !
        error = max(error6,error5,error4)
        !
        deallocate(temp_mat,temp1_mat,stat=err)
        !
        if (err /= 0) then
          !
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'In subroutine init_invs'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'cannot deallocate memory for temp_mat or temp1_mat'
          call catch_exception(0)
          !
        end if
        !
      case(5)
        !
        allocate(temp_mat(5,5),stat=err)
        !
        if (err /= 0) then
          !
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'In subroutine init_invs case(5)'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'cannot allocate memory for temp_mat'
          call catch_exception(0)
          !
        end if
        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        !!!! 5-point functions
        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        origine_inv_info_par = '5x5 matrix'
        call inverse(s_mat,invs5(:,:,1),error5)
        b5(:,1) = sum(invs5(:,:,1),dim=1)
        sumb5(1) = sum(b5(:,1))
        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        !!!! 4-point functions
        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        error4 = 0._ki
        !
        do i1=1,5
          !
          call put_to_zero(i1,s_mat,temp_mat)
          origine_inv_info_par = '4x4 matrix, pinch '//achar(i1+48)
          !~ call inverse(temp_mat,invs4(:,:,1,i1),error)
          call inverse(temp_mat,invs4(:,:,1,i1),error,i1)
          !
          b4(:,1,i1) = sum(invs4(:,:,1,i1),dim=1)
          sumb4(1,i1) = sum(b4(:,1,i1))
          norma_sumb4(1,i1) = sumb4(1,i1)*plus_grand
          error4 = max(error,error4)
          !
        end do
        !
        error = max(error5,error4)
        !
        deallocate(temp_mat,stat=err)
        !
        if (err /= 0) then
          !
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'In subroutine init_invs case(5)'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'cannot deallocate memory for temp_mat'
          call catch_exception(0)
          !
        end if
        !
      case(4)
        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        !!!! 4-point functions
        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        origine_inv_info_par = '4x4 matrix'
        call inverse(s_mat,invs4(:,:,1,1),error4)
        b4(:,1,1) = sum(invs4(:,:,1,1),dim=1)
        sumb4(1,1) = sum(b4(:,1,1))
        norma_sumb4(1,1) = sumb4(1,1)*plus_grand
        !
        error = error4
        !
      case default
        !
        tab_erreur_par(1)%a_imprimer = .true.
        tab_erreur_par(1)%chaine = 'In subroutine init_invs'
        tab_erreur_par(2)%a_imprimer = .true.
        tab_erreur_par(2)%chaine = 'the dimension of the S matrix is not&
                          & correct %d0'
        tab_erreur_par(2)%arg_int = dim_s
        call catch_exception(0)
        !
      end select
      !
    end subroutine init_invs
    !
    !****if* src/kinematic/inversion/put_to_zero
    ! NAME
    !
    !  Subroutine put_to_zero
    !
    ! USAGE
    !
    !  call put_to_zero(i,mati,matf)
    !
    ! DESCRIPTION
    !
    ! This routine put to 0 the line and the raw i of the square matrix mati
    ! It returns a square matrix matf of dim n x n (n being the 
    ! dimension of mati)
    !
    ! INPUTS
    !
    !  * i -- an integer, the value of the line/raw to be put to zero
    !  * mati -- an real (type ki) array of rank 2
    !
    ! SIDE EFFECTS
    !
    !  No side effect
    !
    ! RETURN VALUE
    !
    !  * matf -- an real (type ki) array of rank 2
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    subroutine put_to_zero(i,mati,matf)
      !
      integer, intent(in) :: i
      real(ki), dimension(:,:), intent(in) :: mati
      real(ki), dimension(size(mati,1),size(mati,2)), intent(out) :: matf
      !
      integer :: n
      !
      n = size(mati,1)          ! la matrice mati est carree
      matf = mati
      matf(i,:) = 0._ki
      matf(:,i) = 0._ki
      !
    end subroutine put_to_zero
    !
    !****f* src/kinematic/inversion/inv_s
    ! NAME
    !
    !  Function inv_s
    !
    ! USAGE
    !
    !  real = inv_s(i,j,set)
    !
    ! DESCRIPTION
    !
    !  This function gives the generic inverse of the S matrix whatever 
    !  its dimension (<=6)
    !
    ! INPUTS
    !
    !  * i -- an integer, line number
    !  * j -- an integer, row number
    !  * set -- an integer array of rank 1, the set of pinch propagators
    !
    ! SIDE EFFECTS
    !
    !  No side effect
    !
    ! RETURN VALUE
    !
    !  A real (type ki)
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    function inv_s(i,j,b_pin)
      !
      integer, intent (in) :: i,j
      integer, intent (in) :: b_pin
      real(ki) :: inv_s
      !
      integer :: k,l
      integer, dimension (:), allocatable :: set
      integer :: dim_set
      !
      dim_set = countb(b_pin)
      !
      if (dim_set /= 0) then
        !
        allocate(set(1:dim_set))
        set = unpackb(b_pin,dim_set)
        !
      end if
      !
      select case(dim_s)
      !
      case(6)           ! case where we start with a 6-point amplitude
        !
        if (dim_set == 0) then
          !
          inv_s = invs6(i,j)
          !
        else if (dim_set == 1) then
          !
          k = set(1)
          !
          if ( (i == k) .or. (j == k) ) then
            !
            inv_s = 0._ki
            !
          else
            !
            inv_s = invs5(i,j,k)
            !
          end if
          !
        else if (dim_set == 2) then
          !
          call tri_int(set,set)
          !
          k = set(1)
          l = set(2)
          !
          if ( ( (i == k) .or. (j == k) ) .or. ( (i == l) .or. (j == l) ) ) then
            !
            inv_s = 0._ki
            !
          else
            !
            inv_s = invs4(i,j,k,l)
            !
          end if
          !
        else
          !
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'In function inv_s, for 6-point'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'the array set has not the right dimension: %d0'
          tab_erreur_par(2)%arg_int = dim_set
          call catch_exception(0)
          !
        end if
        !
      case(5)                   ! case where we start with a 5-point amplitude
        !
        if (dim_set == 0) then
          !
          inv_s = invs5(i,j,1)
          !
        else if (dim_set == 1) then
          !
          k = set(1)
          !
          if ( (i == k) .or. (j == k) ) then
            !
            inv_s = 0._ki
            !
          else
            !
            inv_s = invs4(i,j,1,k)
            !
          end if
          !
        else
          !
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'In function inv_s, for 5-point'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'the array set has not the right dimension: %d0'
          tab_erreur_par(2)%arg_int = dim_set
          call catch_exception(0)
          !
        end if
        !
      case(4)                   ! case where we start with a 4-point amplitude
        !
        if (dim_set == 0) then
          !
          inv_s = invs4(i,j,1,1)
          !
        else
          !
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'In function inv_s, for 4-point'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'the array set has not the right dimension: %d0'
          tab_erreur_par(2)%arg_int = dim_set
          call catch_exception(0)
          !
        end if
        !
      case default
        !
        tab_erreur_par(1)%a_imprimer = .true.
        tab_erreur_par(1)%chaine = 'In function invs'
        tab_erreur_par(2)%a_imprimer = .true.
        tab_erreur_par(2)%chaine = 'the dimension of the S matrix is not&
                          & correct %d0'
        tab_erreur_par(2)%arg_int = dim_s
        call catch_exception(0)
        !
      end select
      !
      if (dim_set /= 0) deallocate(set)
      !
    end function inv_s
    !
    !****f* src/kinematic/inversion/hj
    ! NAME
    !
    !  Function hj
    !
    ! USAGE
    !
    !  real = hj(i,j,set)
    !
    ! DESCRIPTION
    !
    ! This function gives the H matrix (pseudo-inverse of G) (dim=6)
    !
    ! INPUTS
    !
    !  * i -- an integer, line number
    !  * j -- an integer, row number
    !  * set -- an integer array of rank 1, the set of pinch propagators
    !
    ! SIDE EFFECTS
    !
    !  No side effect
    !
    ! RETURN VALUE
    !
    !  A real (type ki)
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    function hj(i,j,b_pin)
      !
      integer, intent (in) :: i,j
      integer, intent (in) :: b_pin
      real(ki) :: hj
      !
      integer :: k
      integer, dimension (:), allocatable :: set
      integer :: dim_set
      !
      dim_set = countb(b_pin)
      !
      if (dim_set /= 0) then
        !
        allocate(set(1:dim_set))
        set = unpackb(b_pin,dim_set)
        !
      end if
      !
      select case(dim_s)
      !
      case(6)                   ! case where we start with a 6-point amplitude
        !
        if (dim_set == 1) then
          !
          k = set(1)
          !
          if ( (i == k) .or. (j == k) ) then
            !
            hj = 0._ki
            !
          else
            !
            hj = hjj(i,j,k)
            !
          end if
          !
        else
          !
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'In function hj, for 6-point'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'the array set has not the right dimension: %d0'
          tab_erreur_par(2)%arg_int = dim_set
          call catch_exception(0)
          !
        end if
        !
      case default
        !
        tab_erreur_par(1)%a_imprimer = .true.
        tab_erreur_par(1)%chaine = 'In function hj'
        tab_erreur_par(2)%a_imprimer = .true.
        tab_erreur_par(2)%chaine = 'the dimension of the S matrix is not&
                          & correct %d0'
        tab_erreur_par(2)%arg_int = dim_s
        call catch_exception(0)
        !
      end select
      !
      if (dim_set /= 0) deallocate(set)
      !
    end function hj
    !
    !****f* src/kinematic/inversion/b
    ! NAME
    !
    !  Function b
    !
    ! USAGE
    !
    !  real = b(i,set)
    !
    ! DESCRIPTION
    !
    ! This function gives the b coefficients whatever the S matrix dimension (<=6)
    !
    ! INPUTS
    !
    !  * i -- an integer, label of the b coefficients
    !  * set -- an integer array of rank 1, the set of pinch propagators
    !
    ! SIDE EFFECTS
    !
    !  No side effect
    !
    ! RETURN VALUE
    !
    !  A real (type ki)
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    function b(i,b_pin)
      !
      integer, intent (in) :: i
      integer, intent (in) :: b_pin
      real(ki) :: b
      !
      integer :: k,l
      integer, dimension (:), allocatable :: set
      integer :: dim_set
      !
      dim_set = countb(b_pin)
      !
      if (dim_set /= 0) then
        !
        allocate(set(1:dim_set))
        set = unpackb(b_pin,dim_set)
        !
      end if
      !
      select case(dim_s)
      !
      case(6)                   ! case where we start with a 6-point amplitude
        !
        if (dim_set == 0) then
          !
          b = b6(i)
          !
        else if (dim_set == 1) then
          !
          k = set(1)
          !
          if (i == k) then
            !
            b = 0._ki
            !
          else
            !
            b = b5(i,k)
            !
          end if
          !
        else if (dim_set == 2) then
          !
          call tri_int(set,set)
          k = set(1)
          l = set(2)
          !
          if ( (i == k) .or. (i == l) ) then
            !
            b = 0._ki
            !
          else
            !
            b = b4(i,k,l)
            !
          end if
          !
        else
          !
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'In function b, for 6-point'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'the array set has not the right dimension: %d0'
          tab_erreur_par(2)%arg_int = dim_set
          call catch_exception(0)
          !
        end if
        !
      case(5)                   ! case where we start with a 5-point amplitude
        !
        if (dim_set == 0) then
          !
          b = b5(i,1)
          !
        else if (dim_set == 1) then
          !
          k = set(1)
          !
          if (i == k) then
            !
            b = 0._ki
            !
          else
            !
            b = b4(i,1,k)
            !
          end if
          !
        else
          !
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'In function b, for 5-point'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'the array set has not the right dimension: %d0'
          tab_erreur_par(2)%arg_int = dim_set
          call catch_exception(0)
          !
        end if
        !
      case(4)                   ! case where we start with a 4-point amplitude
        !
        if (dim_set == 0) then
          !
          b = b4(i,1,1)
          !
        else
          !
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'In function b, for 4-point'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'the array set has not the right dimension: %d0'
          tab_erreur_par(2)%arg_int = dim_set
          call catch_exception(0)
          !
        end if
        !
      case default
        !
        tab_erreur_par(1)%a_imprimer = .true.
        tab_erreur_par(1)%chaine = 'In function b'
        tab_erreur_par(2)%a_imprimer = .true.
        tab_erreur_par(2)%chaine = 'the dimension of the S matrix is not&
                          & correct %d0'
        tab_erreur_par(2)%arg_int = dim_s
        call catch_exception(0)
        !
      end select
      !
      if (dim_set /= 0) deallocate(set)
      !
    end function b
    !
    !****f* src/kinematic/inversion/sumb
    ! NAME
    !
    !  Function sumb
    !
    ! USAGE
    !
    !  real = sumb(set)
    !
    ! DESCRIPTION
    !
    ! This function gives the B coefficient whatever the S matrix dimension (<=6)
    !
    ! INPUTS
    !
    !  * set -- an integer array of rank 1, the set of pinch propagators
    !
    ! SIDE EFFECTS
    !
    !  No side effect
    !
    ! RETURN VALUE
    !
    !  A real (type ki)
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    function sumb(b_pin)
      !
      integer, intent (in) :: b_pin
      real(ki) :: sumb
      !
      integer :: k,l
      integer, dimension (:), allocatable :: set
      integer :: dim_set
      !
      dim_set = countb(b_pin)
      !
      if (dim_set /= 0) then
        !
        allocate(set(1:dim_set))
        set = unpackb(b_pin,dim_set)
        !
      end if
      !
      select case(dim_s)
      !
      case(6)                   ! case where we start with a 6-point amplitude
        !
        if (dim_set == 0) then
          !
          sumb = sumb6
          !
        else if (dim_set == 1) then
          !
          k = set(1)
          sumb = sumb5(k)
          !
        else if (dim_set == 2) then
          !
          call tri_int(set,set)
          k = set(1)
          l = set(2)
          sumb = sumb4(k,l)
          !
        else
          !
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'In function sumb, for 6-point'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'the array set has not the right dimension: %d0'
          tab_erreur_par(2)%arg_int = dim_set
          call catch_exception(0)
          !
        end if
        !
      case(5)                   ! case where we start with a 5-point amplitude
        !
        if (dim_set == 0) then
          !
          sumb = sumb5(1)
          !
        else if (dim_set == 1) then
          !
          k = set(1)
          sumb = sumb4(1,k)
          !
        else
          !
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'In function sumb, for 5-point'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'the array set has not the right dimension: %d0'
          tab_erreur_par(2)%arg_int = dim_set
          call catch_exception(0)
          !
        end if
        !
      case(4)                   ! case where we start with a 4-point amplitude
        !
        if (dim_set == 0) then
          !
          sumb = sumb4(1,1)
          !
        else
          !
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'In function sumb, for 4-point'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'the array set has not the right dimension: %d0'
          tab_erreur_par(2)%arg_int = dim_set
          call catch_exception(0)
          !
        end if
        !
      case default
        !
        tab_erreur_par(1)%a_imprimer = .true.
        tab_erreur_par(1)%chaine = 'In function sumb'
        tab_erreur_par(2)%a_imprimer = .true.
        tab_erreur_par(2)%chaine = 'the dimension of the S matrix is not&
                          & correct %d0'
        tab_erreur_par(2)%arg_int = dim_s
        call catch_exception(0)
        !
      end select
      !
      if (dim_set /= 0) deallocate(set)
      !
    end function sumb
    !
    !****f* src/kinematic/inversion/norma_sumb
    ! NAME
    !
    !  Function norma_sumb
    !
    ! USAGE
    !
    !  real = norma_sumb(set)
    !
    ! DESCRIPTION
    !
    ! This function gives the B coefficient whatever the S matrix dimension (<=6)
    ! divided by the greatest (in absolute value) element of the S matrix
    !
    ! INPUTS
    !
    !  * set -- an integer array of rank 1, the set of pinch propagators
    !
    ! SIDE EFFECTS
    !
    !  No side effect
    !
    ! RETURN VALUE
    !
    !  A real (type ki)
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    function norma_sumb(b_pin)
      !
      integer, intent (in) :: b_pin
      real(ki) :: norma_sumb
      !
      integer :: k,l
      integer, dimension (:), allocatable :: set
      integer :: dim_set
      !
      dim_set = countb(b_pin)
      !
      if (dim_set /= 0) then
        !
        allocate(set(1:dim_set))
        set = unpackb(b_pin,dim_set)
        !
      end if
      !
      select case(dim_s)
      !
      case(6)                   ! case where we start with a 6-point amplitude
        !
        if (dim_set == 2) then
          !
          !
          call tri_int(set,set)
          k = set(1)
          l = set(2)
          norma_sumb = norma_sumb4(k,l)
          !
          !
        else
          !
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'In function norma_sumb, for 6-point'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'the array set has not the right dimension: %d0'
          tab_erreur_par(2)%arg_int = dim_set
          call catch_exception(0)
          !
        end if
        !
      case(5)                   ! case where we start with a 5-point amplitude
        !
        if (dim_set == 1) then
          !
          k = set(1)
          norma_sumb = norma_sumb4(1,k)
          !
        else
          !
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'In function norma_sumb, for 5-point'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'the array set has not the right dimension: %d0'
          tab_erreur_par(2)%arg_int = dim_set
          call catch_exception(0)
          !
        end if
        !
      case(4)                   ! case where we start with a 4-point amplitude
        !
        if (dim_set == 0) then
          !
          norma_sumb = norma_sumb4(1,1)
          !
        else
          !
          tab_erreur_par(1)%a_imprimer = .true.
          tab_erreur_par(1)%chaine = 'In function norma_sumb, for 4-point'
          tab_erreur_par(2)%a_imprimer = .true.
          tab_erreur_par(2)%chaine = 'the array set has not the right dimension: %d0'
          tab_erreur_par(2)%arg_int = dim_set
          call catch_exception(0)
          !
        end if
        !
      case default
        !
        tab_erreur_par(1)%a_imprimer = .true.
        tab_erreur_par(1)%chaine = 'In function norma_sumb'
        tab_erreur_par(2)%a_imprimer = .true.
        tab_erreur_par(2)%chaine = 'the dimension of the S matrix is not&
                          & correct %d0'
        tab_erreur_par(2)%arg_int = dim_s
        call catch_exception(0)
        !
      end select
      !
      if (dim_set /= 0) deallocate(set)
      !
    end function norma_sumb
    !
end module matrice_s
