!
!****h* src/module/parametre
! NAME
!
!  Module parametre
!
! USAGE
!
!  use parametre
!
! DESCRIPTION
!
!  This module is used to pass some variables used by many functions of 
!  the GOLEM program. Note that these variables can be rewritten.
!  It contains also a routine to print the parameters (it prints on unit 6)
!
! OUTPUT
!
!  It exports the variables:
!  * tolerance -- a real (type ki), the tolerance for the numerical integration
!  * lambda_par -- a real (type ki), a parameter of the contour deformation
!  * alpha_par -- a real (type ki), a parameter of the contour deformation
!  * beta_par -- a real (type ki), a parameter of the contour deformation
!  * coupure_3p2m -- a real (type ki), a cut between numerical and analytical 
!                    computation for two mass three point functions
!  * coupure_3p3m -- a real (type ki), a cut between numerical and analytical 
!                    computation for three mass three point functions
!  * coupure_4p1m -- a real (type ki), a cut between numerical and analytical 
!                    computation for one mass four point functions
!  * coupure_4p2m_opp -- a real (type ki), a cut between numerical and analytical 
!                    computation for two mass opposite four point functions
!  * coupure_4p2m_adj -- a real (type ki), a cut between numerical and analytical 
!                    computation for two mass adjacent four point functions
!  * coupure_4p3m -- a real (type ki), a cut between numerical and analytical 
!                    computation for three mass four point functions
!  * coupure_4p4m -- a real (type ki), a cut between numerical and analytical 
!                    computation for four mass four point functions (not active)
!  * coupure_3p2m_1mi --  a real (type ki), a cut between numerical and analytical 
!                    computation for one internal mass two external mass three point functions
!  * rat_or_tot_par -- a character (len=3)
!  * if_print_info_par -- a logical, if true it prints some informations concerning the numerical
!                    integration
!  * if_print_warn_par -- a logical, if true it prints some informations concerning the warning 
!                    about numerical precision
!  * accuracy_par -- the accuracy for the matrix inversion and the numerical integration
!  * not_enough_accuracy_par -- a flag to ring the bell if the accuracy is not reached in
!                               in the matrix inversion and the numerical integration
!  * mu2_scale_par -- the square of the renormalisation scale
!  * subroutine print_parameter -- to print these variables
!
! USES
!
!  * precision (src/module/preci_double.f90)
!
!*****
module parametre
  use precision
  implicit none
  !
  real(ki),save :: tolerance = 1.e-8_ki  ! precision for Gaussian integration
  !
  real(ki),save :: lambda_par = 1._ki  ! parameters of the contour
  real(ki),save :: alpha_par = 1._ki   ! deformation
  real(ki),save :: beta_par = 1._ki    !
  !
  real(ki),save :: coupure_3p2m = 5.e-3_ki ! value to switch between analytical and numerical
                                           ! evaluation for two mass three point functions
  real(ki),save :: coupure_3p3m = 5.e-3_ki ! value to switch between analytical and numerical
                                           ! evaluation for three mass three point functions
  real(ki),save :: coupure_4p1m = 5.e-3_ki ! value to switch between analytical and numerical
                                           ! evaluation for one/zero mass four point functions
  real(ki),save :: coupure_4p2m_opp = 5.e-3_ki ! value to switch between analytical and numerical
                                               ! evaluation for two opposite mass four point functions
  real(ki),save :: coupure_4p2m_adj = 5.e-3_ki! value to switch between analytical and numerical
                                              ! evaluation for two adjacent mass four point functions
  real(ki),save :: coupure_4p3m = 5.e-3_ki ! value to switch between analytical and numerical
                                           ! evaluation for three mass four point functions
  real(ki),save :: coupure_4p4m = 0._ki ! value to switch between analytical and numerical
                                        ! evaluation for four mass four point functions
  real(ki),save :: coupure_3p2m_1mi = 5.e-3_ki ! value to switch between analytical and numerical
                                        ! evaluation for one internal mass two external mass three point functions
  !~ character (len=3),save :: rat_or_tot_par = 'rat'
  character (len=3),save :: rat_or_tot_par = 'tot'
  logical, save :: if_print_info_par = .false. ! if true print information
  logical, save :: if_print_warn_par = .false. ! if true print information for warning
  real(ki),save :: accuracy_par = 1.e-10_ki ! the accuracy for the matrix inversion and the numerical integration
  logical, save :: not_enough_accuracy_par = .false. !
  real(ki), save :: mu2_scale_par = 1._ki ! the square of the renormalisation scale
  logical, save :: ffini_unknown = .true.
  !
 contains
    !
    !****f* src/module/print_parameter
    ! NAME
    !
    !  Subroutine print_parameter
    !
    ! USAGE
    !
    !  call print_parameter()
    !
    ! DESCRIPTION
    !
    !  This routine print the variables defined in the module parametre
    !
    ! INPUTS
    !
    !  No inputs
    !
    ! SIDE EFFECTS
    !
    !  No side effect
    !
    ! RETURN VALUE
    !
    !  It prints on the unit 6
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    subroutine print_parameter()
      !
      integer :: unit
      !
      unit = 6
      !
      write(unit,*) 'tolerance :',tolerance
      write(unit,*) 'lambda_par :',lambda_par
      write(unit,*) 'alpha_par :',alpha_par
      write(unit,*) 'beta_par :',beta_par
      write(unit,*) 'coupure_3p2m :',coupure_3p2m
      write(unit,*) 'coupure_3p3m :',coupure_3p3m
      write(unit,*) 'coupure_4p1m :',coupure_4p1m
      write(unit,*) 'coupure_4p2m_opp :',coupure_4p2m_opp
      write(unit,*) 'coupure_4p2m_adj :',coupure_4p2m_adj
      write(unit,*) 'coupure_4p3m :',coupure_4p3m
      write(unit,*) 'coupure_4p4m :',coupure_4p4m
      write(unit,*) 'rat_or_tot_par : ',rat_or_tot_par
      write(unit,*) 'if_print_info_par : ',if_print_info_par
      write(unit,*) 'if_print_warn_par : ',if_print_warn_par
      write(unit,*) 'accuracy_par : ',accuracy_par
      write(unit,*) 'not_enough_accuracy_par : ',not_enough_accuracy_par
      write(unit,*) 'mu2_scale_par : ',mu2_scale_par
      write(unit,*) 'accuracy_par : ',accuracy_par
      write(unit,*) 'not_enough_accuracy_par : ',not_enough_accuracy_par
      write(unit,*) 'mu2_scale_par : ',mu2_scale_par
      !
    end subroutine print_parameter
    !
end module parametre
