! 
!****h* src/module/tri_croissant
! NAME
!
!  Module tri_croissant
!
! USAGE
!
!  use tri_croissant
!
! DESCRIPTION
!
!  This module is used to sort an integer array or shift its elements by certain amount
!
! OUTPUT
!
!  This module exports:
!  * tri_int -- a subroutine to sort out an integer array
!  * shift_param -- a subroutine to shift (modulo n) the elements of an integer array
!
! USES
!
!  * precision (src/module/preci_double.f90)
!
!*****
module tri_croissant
  !
  use precision
  implicit none
  !
  private
  integer, dimension(:), allocatable :: temp_glob
  !
  integer, parameter :: tri_max = 100
  integer, dimension(0:tri_max) :: temp_glob_new
  !
  public :: tri_int,shift_param,exchange_param
  !
  contains
    !
    !****f* src/module/tri_croissant/tri_int
    ! NAME
    !
    !  Subroutine tri_int
    !
    ! USAGE
    !
    !  call tri_int(t_in,t_out)
    !
    ! DESCRIPTION
    !
    !  This routine sorts in increasing order an integer array t_int and put the 
    !  result in the integer array t_out
    !
    ! INPUTS
    !
    !  * t_int -- an integer array of rank 1, the array to sort
    !
    ! SIDE EFFECTS
    !
    !  modify the private variable temp_glob, used by the routine exchange
    !
    ! RETURN VALUE
    !
    !  * t_out -- an integer array of rank 1, the sorted array
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    ! on trie par ordre croissant le tableau t_in et 
    ! on met le resultat dans t_out. On utilise les procedures 
    ! compare et exchange
    subroutine tri_int(t_in,t_out)
      !
      integer, intent(in), dimension(:) :: t_in
      integer, intent(out), dimension(:) :: t_out
      !
      integer :: j,i
      !
      allocate(temp_glob(size(t_in)))
      temp_glob = t_in
      !
      do j = 2,size(t_in)
        !
        i = j - 1
        !
        do while (i >= 1 .and. compare(temp_glob(i),temp_glob(i+1)))
          !
          call exchange(i,i+1)  ! deplace l'element i+1 a la place de i
          i = i - 1
          if (i == 0) exit ! to avoid the do-while test when i=0
          !
        end do
        !
      end do
      !
      t_out = temp_glob
      deallocate(temp_glob)
      !
    end subroutine tri_int
    !
    subroutine tri_int_new(t_in, t_out)
      integer, intent(in), dimension(:) :: t_in
      integer, intent(out), dimension(:) :: t_out
      !
      ! Since the arrays we want to sort are small and the entries
      ! are distributed very dense a simple bucket-sort might do a better job.
      ! The only assumptions put in is that all elements t_in(i) are 0 <= t_in(i) <= tri_max.
      integer :: i, j, ti, l
      !
      l = size(t_in)
      temp_glob_new(:) = 0
      !
      do i = 1, l
        !
        temp_glob_new(i) = temp_glob_new(i) + 1
        !
      end do
      !
      ti = 0
      !
      do i = 1, tri_max
        !
        do j = 1, temp_glob_new(i)
          !
          ti = ti + 1
          t_out(ti) = i
          !
        end do
        !
        ! avoid running into the trailing zeroes
        if (ti == l) exit
        !
      end do
      !
    end subroutine tri_int_new
    !
    !****if* src/module/tri_croissant/compare
    ! NAME
    !
    !  Function compare
    !
    ! USAGE
    !
    !  logical = compare(i,j)
    !
    ! DESCRIPTION
    !
    !  This function returs true if i > j
    !
    ! INPUTS
    !
    !  * i -- an integer
    !  * j -- an integer
    !
    ! SIDE EFFECTS
    !
    !  no side effect
    !
    ! RETURN VALUE
    !
    !  it returns a logical
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    ! cette fonction retourne vrai si i > j
    function compare(i,j)
      !
      integer, intent(in) :: i,j
      logical :: compare
      !
      compare = .false.
      if (i > j) compare = .true.
      !
    end function compare
    !
    !****if* src/module/tri_croissant/exchange
    ! NAME
    !
    !  Subroutine exchange
    !
    ! USAGE
    !
    !  call exchange(i,j)
    !
    ! DESCRIPTION
    !
    !  This routine exchanges the arguments i and j of the private array temp_glob
    !
    ! INPUTS
    !
    !  * i -- an integer
    !  * j -- an integer
    !
    ! SIDE EFFECTS
    !
    !  modify the private array temp_glob
    !
    ! RETURN VALUE
    !
    !  No return value
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    ! cette subroutine echange les elements i et j du tableau temp_glob
    subroutine exchange(i,j)
      !
      integer, intent(in) :: i,j
      !
      integer :: truc
      !
      truc = temp_glob(j)
      temp_glob(j) = temp_glob(i)
      temp_glob(i) = truc
      !
    end subroutine exchange
    !
    !****f* src/module/tri_croissant/shift_param
    ! NAME
    !
    !  Subroutine shift_param
    !
    ! USAGE
    !
    !  call shift_param(z_param_ini,shift,modd,z_param_out)
    !
    ! DESCRIPTION
    !
    !  This routine shifts the array z_param_ini of Feynman parameters
    !  the shift is done as following: 
    !  z --> z+shift if z+shift <= modd 
    !  else mod(z+shift-1,modd)+1 if z+shift > modd
    !  the result is put into the array z_param_out
    !
    ! INPUTS
    !
    !  * z_param_ini -- an integer array of rank 1, the array to shift
    !  * shift -- an integer, the value of the shift
    !  * modd -- an integer, the shift is made modulo modd
    !
    ! SIDE EFFECTS
    !
    !  No side effect
    !
    ! RETURN VALUE
    !
    !  * z_param_out -- an integer array of rank 1, the shifted array
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    subroutine shift_param(z_param_ini,shift,modd,z_param_out)
      !
      integer, intent(in) :: shift,modd
      integer, intent(in), dimension(modd) :: z_param_ini
      integer, intent(out), dimension(modd) :: z_param_out
      !
      integer, dimension(modd) :: temp
      integer :: i,tt
      !
      do i=1,modd
        !
        tt = z_param_ini(i)
        !
        if (tt /= 0) then
          !
          tt = tt + shift - 1
          tt = modulo(tt,modd)
          tt = tt + 1
          !
        end if
        !
        temp(i) = tt
        !
      end do
      !
      call tri_int(temp,z_param_out)
      !
    end subroutine shift_param
    !
    !
    !****f* src/module/tri_croissant/exchange_param
    ! NAME
    !
    !  Subroutine exchange_param
    !
    ! USAGE
    !
    !  call exchange_param(z_param_ini,tab,modd,z_param_out)
    !
    ! DESCRIPTION
    !
    !  This routine exchanges in the array z_param_ini of Feynman parameters
    !  the label tab(1) with the label tab(2)
    !  the result is put into the array z_param_out
    !
    ! INPUTS
    !
    !  * z_param_ini -- an integer array of rank 1, the array to shift
    !  * tab -- an integer array of rank 1, the two labels to exchange
    !  * modd -- an integer, the shift is made modulo modd
    !
    ! SIDE EFFECTS
    !
    !  No side effect
    !
    ! RETURN VALUE
    !
    !  * z_param_out -- an integer array of rank 1, the shifted array
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    subroutine exchange_param(z_param_ini,tab,modd,z_param_out)
      !
      integer, intent(in) :: modd
      integer, dimension(2), intent(in) :: tab
      integer, intent(in), dimension(modd) :: z_param_ini
      integer, intent(out), dimension(modd) :: z_param_out
      !
      integer, dimension(modd) :: temp
      integer :: i,tt
      !
      do i=1,modd
        !
        tt = z_param_ini(i)
        !
        if (tt == tab(1) ) then
          !
          tt = tab(2)
          !
        else if (tt == tab(2) ) then
          !
          tt = tab(1)
          !
        end if
        !
        temp(i) = tt
        !
      end do
      !
      call tri_int(temp,z_param_out)
      !
    end subroutine exchange_param
    !
end module tri_croissant
!
