!****h* src/module/logarithme
! NAME
!
!  Module logarithme
!
! USAGE
!
!  use logarithme
!
! DESCRIPTION
!
!  This module provides three public routines to compute the logarithm,
!  the logarithm squared and a specila function (generalisation of ln(1-z)/z) 
!  assumig that the argument is of the type z = a + i lambda s where a is a real
!  lambda > 0 and << 1 and s = +/- 1
!
! OUTPUT
!
!  It exports:
!  * z_log -- a function which returns the logarithm with a complex argument
!  * z_log2 -- a function which returns the logarithm squared with a complex argument
!  * q -- a recursive function which a generalisation of ln(1-z)/z
!
! USES
!
!  * precision (src/module/preci_double.f90)
!  * constante (src/module/constante.f90)
!  * sortie_erreur (src/module/sortie_erreur.f90)
!
!*****
module logarithme
  !
  use precision
  use constante, only : pi,i_,un
  use sortie_erreur, only : tab_erreur_par,catch_exception
  implicit none
  !
  private
  real(ki), parameter :: small_glob = 5.e-1_ki
  public :: z_log,z_log2,q
  !
  contains
    !
    !****f* src/module/logarithme/z_log
    ! NAME
    !
    !  Function z_log
    !
    ! USAGE
    !
    !  complex = z_log(a,s)
    !
    ! DESCRIPTION
    !
    !  Compute the ln(z) with z = a + i lambda s
    !
    ! INPUTS
    !
    !  * a -- a real (type ki), the real part
    !  * s -- a real (type ki), s = +/- 1, it gives the sign of the small imaginary part
    !
    ! SIDE EFFECTS
    !
    !  No side effect
    !
    ! RETURN VALUE
    !
    !  This function returns a complex (type ki)
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    function z_log(a,s)
      !
      real(ki), intent(in) :: a,s
      complex(ki) :: z_log
      !
      if (abs(s) == 1._ki) then
        !
        if (a > 0._ki) then
          !
          z_log = log(a)
          !
        else
          !
          z_log = log(-a)+i_*pi*s
          !
        endif
        !
      else
        !
        tab_erreur_par(1)%a_imprimer = .true.
        tab_erreur_par(1)%chaine = 'error in z_log:'
        tab_erreur_par(2)%a_imprimer = .true.
        tab_erreur_par(2)%chaine = 'the second argument must be 1. or -1. %f0'
        tab_erreur_par(2)%arg_real = s
        call catch_exception(0)
        !
      endif
      !
    end function z_log
    !
    !****f* src/module/logarithme/z_log2
    ! NAME
    !
    !  Function z_log2
    !
    ! USAGE
    !
    !  complex = z_log2(a,s)
    !
    ! DESCRIPTION
    !
    !  Compute the ln(z)^2 with z = a + i lambda s
    !
    ! INPUTS
    !
    !  * a -- a real (type ki), the real part
    !  * s -- a real (type ki), s = +/- 1, it gives the sign of the small imaginary part
    !
    ! SIDE EFFECTS
    !
    !  No side effect
    !
    ! RETURN VALUE
    !
    !  This function returns a complex (type ki)
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    function z_log2(a,s)
      !
      real(ki), intent(in) :: a,s
      complex(ki) :: z_log2
      !
      if (abs(s) == 1._ki) then
        !
        if (a > 0._ki) then
          !
          z_log2 = log(a)**2
          !
        else
          !
          z_log2 = log(-a)**2-pi**2+2._ki*pi*i_*s*log(-a)
          !
        endif
        !
      else
        !
        tab_erreur_par(1)%a_imprimer = .true.
        tab_erreur_par(1)%chaine = 'error in z_log2:'
        tab_erreur_par(2)%a_imprimer = .true.
        tab_erreur_par(2)%chaine = 'the second argument must be 1. or -1. %f0'
        tab_erreur_par(2)%arg_real = s
        call catch_exception(0)
        !
      endif
      !
    end function z_log2
    !
    !****f* src/module/logarithme/q
    ! NAME
    !
    !  Function q
    !
    ! USAGE
    !
    !  complex = q(n,x,s)
    !
    ! DESCRIPTION
    !
    !  It computes the function q defined recusively by
    !  q_n(X) = (q_{n-1}(X)+1/(n-1))/X 
    !  with q_1(X) = ln(1-X)/X 
    !  assuming that X = x + i*s*lambda and s=+/- 1, 
    !  Care is taken for small values of x.
    !  For x < small_glob
    !  q_n(x) = -( 1/n + \sum_{j=n+1}^\infinity x^{j-n}/j )
    !  Note that in this case there is no imaginary part.
    !
    ! INPUTS
    !
    !  * n -- an integer, the order of q
    !  * x -- a real (type ki), the real part
    !  * s -- a real (type ki), s = +/- 1, it gives the sign of the small imaginary part
    !
    ! SIDE EFFECTS
    !
    !  No side effect
    !
    ! RETURN VALUE
    !
    !  This function returns a complex (type ki)
    !
    ! EXAMPLE
    !
    !
    !
    !*****
    !
    recursive function q(n,x,s) result(resq)
      !
      integer, intent(in) :: n
      real(ki), intent(in) :: x,s
      complex(ki) :: resq
      !
      integer :: nm1
      real(ki) :: tt,temp,expo_x,denom
      !
      if (abs(x) > small_glob) then
        !
        if (n == 1) then
          resq = z_log(un-x,-s)/x
        else
          nm1 = n - 1
          resq = (q(nm1,x,s) + 1._ki/real(nm1,ki))/x
        end if
        !
      else ! no imaginary part in this case
        !
        denom = real(n,ki)
        tt = 1._ki/denom
        expo_x = un
        temp = 10._ki !artificial value to enter into the loop
        !
        do while(abs(tt-temp) >= epsilon(x))
          !
          temp = tt
          expo_x = x*expo_x
          denom = denom + un
          tt = tt + expo_x/denom
          !
        end do
        !
        resq = cmplx(-tt,0._ki,ki)
        !
      end if
      !
    end function q
    !
end module logarithme
